/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include "ndmalloc.h"

struct ItemData {
    size_t magnitude;
    size_t total;
    size_t offset;
    size_t size;
};

void *nd_malloc (size_t size, ...)
{
    va_list list;

    int dimensions = 0;

    {
        va_start (list, size);
        for (size_t current = size; current != 0; current = va_arg (list, size_t)) {
          dimensions++;
        }
        va_end (list);
    }
    dimensions --;

    if (dimensions <= 0) return 0;
    
    struct ItemData *sizes = (struct ItemData *) malloc (sizeof (struct ItemData) * dimensions);
    size_t total = 0;
    {
        va_start (list, size);
        for (int i = 0; i < dimensions; ++i) {

            sizes[i].magnitude = va_arg (list, size_t) + 1;
            if (i == dimensions -1) sizes[i].magnitude--;

            sizes[i].size = (i == dimensions -1)? size : sizeof (void *);

            sizes[i].total = sizes[i].size;
            for (int d = 0; d <= i; ++d) {
                sizes[i].total *= sizes [d].magnitude;
            }

            sizes[i].offset = (i == 0)? 0 : sizes[i-1].offset;
            sizes[i].offset += (i == 0)? 0 : sizes[i-1].total;
            total += sizes[i].total;
#ifdef W
            printf ("item[%d].total = %d .offset = %d .size = %d .magnitude = %d\n", i, sizes[i].total, sizes[i].offset, sizes[i].size, sizes[i].magnitude);
#endif
        }

        va_end (list);
    }

    char *data = (char *)malloc (total);

#ifdef W
    printf ("dimensions = %d\n", dimensions);
#endif
    for (int i = 0; i < dimensions -1; ++i) {
        char **p = (char **) data + sizes[i].offset;
        char *n = (char *) data + sizes[i+1].offset;
#ifdef W
        printf ("magnitude[%d] = %d\n", i, sizes[i].magnitude);
#endif
        for (unsigned d = 0; d < sizes[i].magnitude -1; ++d) {
#ifdef W
            printf ("p[%d] = %d + %d * %d * %d = ", d, n, sizes[i+1].magnitude, sizes[i+1].size, d);
#endif
            p[d] = n + sizes[i+1].magnitude * sizes[i+1].size * d;
#ifdef W
            printf ("%d\n", p [d]);
#endif
        }
        p [sizes[i].magnitude -1] = 0;
    }
    free (sizes);
#ifdef W
    printf ("total = %d\n", total);
#endif
    return data;
}

