/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdarg.h>
#include "Debug.h"

char buffer [256];
unsigned short buffer2 [256];

#ifdef EKA2
#ifdef WINSCW
#ifndef UIQ3
static REcmt *iEcmt = 0;
#include <EcmtClient.h>
void DebugPrintf (const char *fmt, ...)
{
	if (iEcmt == 0) {
		iEcmt = new (ELeave) REcmt ();
		iEcmt->Connect();
	}

	va_list args;
	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);

	TPtr ptr (buffer2, 256);
	for (int i = 0; ((i < 256) && (buffer[i] != '\0')); ++i) {
		ptr.Append ((TChar)(buffer[i]));
	}

	iEcmt->WriteFormat (_L("%S"), &ptr);
	iEcmt->Close ();
	delete iEcmt;
	iEcmt = 0;
}
#endif
#else
#include <e32svr.h>
void DebugPrintf (const char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);

	TPtr ptr (buffer2, 256);
	for (int i = 0; ((i < 256) && (buffer[i] != '\0')); ++i) {
		ptr.Append ((TChar)(buffer[i]));
	}

        RDebug::Print (_L("%S"), &ptr);
}
#endif

#else
#ifdef __WINS__
#include <e32svr.h>
void DebugPrintf (const char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);

	TPtr ptr (buffer2, 256);
	for (int i = 0; ((i < 256) && (buffer[i] != '\0')); ++i) {
		ptr.Append ((TChar)(buffer[i]));
	}

        RDebug::Print (_L("%S"), &ptr);
}
#else // not __WINS__
#ifdef WIN32
void DebugPrintf (const char *fmt, ...)
{
    static FILE *file = 0;

    if (0 == file) {
        file = fopen ("gnugo.log", "w");
    }
    va_list args;
	va_start(args, fmt);
	vfprintf(file, fmt, args);
    fprintf(file, "\n");
	va_end(args);
}
#else
#include <flogger.h>
void DebugPrintf (const char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);

	TPtr ptr (buffer2, 256);
	for (int i = 0; ((i < 256) && (buffer[i] != '\0')); ++i) {
		ptr.Append ((TChar)(buffer[i]));
	}

        static TBool first = ETrue;

        RFileLogger iLog;
        iLog.Connect();
        if (first) {
            iLog.CreateLog(_L("GnuGoS60"),_L("GnuGo60.txt"), EFileLoggingModeOverwrite);
            first = EFalse;
        } else {
            iLog.CreateLog(_L("GnuGoS60"),_L("GnuGo60.txt"), EFileLoggingModeAppend);
        }

        iLog.WriteFormat (_L("%S"), &ptr);
        iLog.CloseLog ();
        iLog.Close ();
}
#endif // WIN32
#endif // __WINS__
#endif
