#ifndef GLOBALS_H
#define GLOBALS_H
/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifdef __cplusplus
extern "C" {
#endif

#ifdef GLOBALS_DLL
#define GLOBALS_EXTERN __declspec (dllexport)
#else
#define GLOBALS_EXTERN __declspec (dllimport)
#endif

#ifdef ENGINE_DLL
#define ENGINE_EXTERN __declspec (dllexport)
#else
#define ENGINE_EXTERN __declspec (dllimport)
#endif

    GLOBALS_EXTERN struct globals* global ();
    GLOBALS_EXTERN void init_globals ();

#include "board.h"
#include "patterns.h"
#include "move_reasons.h"
#include "hash.h"
#include "gg_utils.h"
#include "cache.h"

    struct board_cache_entry {
        int threshold;
        int liberties;
        Hash_data position_hash;
    };
    /* Used in the whose_moyo() function */
    struct moyo_determination_data
    {
        float influence_balance;
        float my_influence_minimum;
        float opp_influence_maximum;
    };
#define MAX_TACTICAL_POINTS 10
#define MAXEYE 20
#define MAX_GOAL_WORMS 15  /* maximum number of worms in a dragon to be */

    /* This is a private array to obtain a list of worms belonging to each
     * dragon. Public access is via first_worm_in_dragon() and
     * next_worm_in_dragon().
     */

    struct worm_data {
        int color;         /* its color */
        int size;          /* its cardinality */
        float effective_size; /* stones and surrounding spaces */
        int origin;        /* the origin of the string. Two vertices are in */
        /* the same worm iff they have same origin. */
        int liberties;     /* number of liberties */
        int liberties2;    /* number of second order liberties */
        int liberties3;    /* third order liberties (empty vertices at distance 3) */
        int liberties4;    /* fourth order liberties */
        int lunch;         /* if lunch != 0 then lunch points to a boundary */
        /* worm which can be captured easily. */
        int cutstone;      /* 1=potential cutting stone; 2=cutting stone */
        int cutstone2;     /* Number of potential cuts involving the worm. */
        int genus;         /* number of connected components of the complement, less one */
        int inessential;   /* 1=inessential worm */
        int invincible;    /* 1=strongly unconditionally non-capturable */
        enum dragon_status unconditional_status; /* ALIVE, DEAD, WHITE_TERRITORY,
                                                    BLACK_TERRITORY, UNKNOWN */

        /* The following arrays keeps track of up to MAX_TACTICAL_POINTS
         * different attack, defense, attack threat, and defense threat
         * points with corresponding result codes. (0 = loss, 1 = bad ko, 2
         * = good ko, 3 = win). The arrays are guaranteed to be sorted with
         * respect to the codes so that the first element contains the best
         * result.
         */
        int attack_points[MAX_TACTICAL_POINTS];
        int attack_codes[MAX_TACTICAL_POINTS];
        int defense_points[MAX_TACTICAL_POINTS];
        int defense_codes[MAX_TACTICAL_POINTS];
        int attack_threat_points[MAX_TACTICAL_POINTS];
        int attack_threat_codes[MAX_TACTICAL_POINTS]; 
        int defense_threat_points[MAX_TACTICAL_POINTS];
        int defense_threat_codes[MAX_TACTICAL_POINTS];
    };
    struct half_eye_data {
        float value;          /* Topological eye value. */
        char type;            /* HALF_EYE or FALSE_EYE; */
        int num_attacks;      /* number of attacking points */
        int attack_point[4];  /* the moves to attack a topological halfeye */
        int num_defenses;     /* number of defending points */
        int defense_point[4]; /* the moves to defend a topological halfeye */
    };
    //ENGINE_EXTERN extern struct worm_data worm[BOARDMAX];
#define MAX_SURROUND 10

    struct surround_data {
        int dragon_number;           /* number of the (surrounded) beast */
        char surround_map[BOARDMAX]; /* surround map                     */
    };
    struct dragon_data {
        int color;    /* its color                                                 */
        int id;       /* the index into the dragon2 array                          */
        int origin;   /* the origin of the dragon. Two vertices are in the same    */
        /* dragon iff they have same origin.                         */
        int size;     /* size of the dragon                                        */
        float effective_size; /* stones and surrounding spaces                     */
        enum dragon_status crude_status; /* (ALIVE, DEAD, UNKNOWN, CRITICAL)       */
        enum dragon_status status;       /* best trusted status                    */
    };
#define MAX_NEIGHBOR_DRAGONS 10

#include "eyes.h"
    struct dragon_data2 {
        int origin;                         /* the origin of the dragon            */
        int adjacent[MAX_NEIGHBOR_DRAGONS]; /* adjacent dragons                    */
        int neighbors;                      /* number of adjacent dragons          */
        int hostile_neighbors;              /* neighbors of opposite color         */

        int moyo_size;		      /* size of surrounding influence moyo, */
        float moyo_territorial_value;       /* ...and its territorial value        */
        enum dragon_status safety;          /* a more detailed status estimate     */
        float weakness; /* A new (3.4) continuous estimate of the dragon's safety  */
        float weakness_pre_owl;     /* Dragon safety based on pre-owl computations */
        int escape_route; /* a measurement of likelihood of escape                 */
        struct eyevalue genus;    /* the number of eyes (approximately)            */
        int heye;     /* coordinates of a half eye                                 */
        int lunch;    /* if lunch != 0 then lunch points to a boundary worm which  */
        /* can be captured easily.                                   */
        int surround_status;         /* Is it surrounded?                          */
        int surround_size;           /* Size of the surrounding area               */

        int semeais;         /* number of semeais in which the dragon is involved  */
        int semeai_defense_point;/* Move found by semeai code to rescue dragon     */
        int semeai_defense_certain;
        int semeai_defense_target; /* The opponent dragon involved in the semeai   */
        int semeai_attack_point; /* Move found by semeai code to kill dragon       */
        int semeai_attack_certain;
        int semeai_attack_target; /* The opponent dragon involved in the semeai    */
        enum dragon_status owl_threat_status; /* CAN_THREATEN_ATTACK/DEFENSE       */
        enum dragon_status owl_status; /* (ALIVE, DEAD, UNKNOWN, CRITICAL, UNCHECKED)    */
        int owl_attack_point;    /* vital point for attack                         */
        int owl_attack_code;     /* ko result code                                 */
        int owl_attack_certain;  /* 0 if owl reading node limit is reached         */
        int owl_second_attack_point;/* if attacker gets both attack points, wins   */
        int owl_defense_point;   /* vital point for defense                        */
        int owl_defense_code;    /* ko result code                                 */
        int owl_defense_certain; /* 0 if owl reading node limit is reached         */
        int owl_second_defense_point;/* if defender gets both attack points, wins  */
        int owl_attack_kworm;    /* only valid when owl_attack_code is GAIN        */
        int owl_defense_kworm;   /* only valid when owl_defense_code is LOSS       */
    };

    /* dragon2 is dynamically allocated */
    //ENGINE_EXTERN extern int number_of_dragons;
    //ENGINE_EXTERN extern struct dragon_data2 *dragon2;

    //ENGINE_EXTERN extern struct dragon_data dragon[BOARDMAX];
    /* We use a forward declaration of influence_data so that the rest
     * of the engine can reference influence data. It can only be accessed
     * in influence.c, however!
     */
    //struct influence_data;
    //ENGINE_EXTERN extern struct influence_data initial_black_influence;
    //ENGINE_EXTERN extern struct influence_data initial_white_influence;
    //ENGINE_EXTERN extern struct influence_data move_influence;
    //ENGINE_EXTERN extern struct influence_data followup_influence;

    //extern struct surround_data surroundings[MAX_SURROUND];
    //extern int surround_pointer;

    /* array of half-eye data */
    //ENGINE_EXTERN extern struct half_eye_data half_eye[BOARDMAX];
    struct aftermath_data {
        int white_captured;
        int black_captured;
        int white_prisoners;
        int black_prisoners;
        int white_territory;
        int black_territory;
        int white_area;
        int black_area;
        int white_control[BOARDMAX];
        int black_control[BOARDMAX];
        enum dragon_status final_status[BOARDMAX];
    };
#define MAX_EYE_ATTACKS 3

    struct eye_data {
        int color;/* BLACK, WHITE, BLACK_BORDERED, WHITE_BORDERED or GRAY_BORDERED */
        int esize;         /* size of the eyespace                                 */
        int msize;         /* number of marginal vertices                          */
        int origin;        /* The origin                                           */
        struct eyevalue value; /* Number of eyes.                                  */
        int attack_point;  /* vital point for attack */
        int defense_point; /* vital point for defense	*/

        /* The above fields are constant on the whole eyespace. */
        /* ---------------------------------------------------------------- */
        /* The below fields are not. */

        char marginal;             /* This vertex is marginal                    */
        char type;                 /* Various characteristics of the eyespace    */
        char neighbors;            /* number of neighbors in eyespace            */
        char marginal_neighbors;   /* number of marginal neighbors               */
    };

    struct vital_eye_points {
        int attack_points[MAX_EYE_ATTACKS];
        int defense_points[MAX_EYE_ATTACKS];
    };

    //extern struct vital_eye_points black_vital_points[BOARDMAX];
    //extern struct vital_eye_points white_vital_points[BOARDMAX];

    //ENGINE_EXTERN extern struct eye_data white_eye[BOARDMAX];
    //ENGINE_EXTERN extern struct eye_data black_eye[BOARDMAX];

    /* Array with the information which was previously stored in the cut
     * field and in the INHIBIT_CONNECTION bit of the type field in struct
     * eye_data.
     */
    //ENGINE_EXTERN extern int cutting_points[BOARDMAX];

    /* Global variables to tune strategy. */

    //extern int use_optimistic_territory;

    //extern float minimum_value_weight;
    //extern float maximum_value_weight;
    //extern float invasion_malus_weight;
    //extern float strategical_weight;
    //extern float territorial_weight;
    //extern float attack_dragon_weight;
    //extern float followup_weight;
    /* ================================================================ */
    /*                         global variables                         */
    /* ================================================================ */

    //extern int disable_threat_computation;
    //extern int disable_endgame_patterns;
    //ENGINE_EXTERN extern int doing_scoring;

    /* Reading parameters */
    //ENGINE_EXTERN extern int depth;               /* deep reading cutoff */
    //ENGINE_EXTERN extern int backfill_depth;      /* deep reading cutoff */
    //ENGINE_EXTERN extern int backfill2_depth;     /* deep reading cutoff */
    //ENGINE_EXTERN extern int break_chain_depth;   /* deep reading cutoff */
    //ENGINE_EXTERN extern int superstring_depth;   /* deep reading cutoff */
    //ENGINE_EXTERN extern int branch_depth;        /* deep reading cutoff */
    //ENGINE_EXTERN extern int fourlib_depth;       /* deep reading cutoff */
    //ENGINE_EXTERN extern int ko_depth;            /* deep ko reading cutoff */
    //ENGINE_EXTERN extern int aa_depth;            /* deep global reading cutoff */
    //extern int depth_offset;        /* keeps track of temporary depth changes */
    //ENGINE_EXTERN extern int owl_distrust_depth;  /* below this owl trusts the optics code */
    //ENGINE_EXTERN extern int owl_branch_depth;    /* below this owl tries only one variation */
    //ENGINE_EXTERN extern int owl_reading_depth;   /* owl does not read below this depth */
    //ENGINE_EXTERN extern int owl_node_limit;      /* maximum number of nodes considered */
    //extern int semeai_branch_depth;
    //extern int semeai_branch_depth2;
    //ENGINE_EXTERN extern int semeai_node_limit;
    //extern int connect_depth;
    //extern int connect_depth2;
    //extern int connection_node_limit;
    //extern int breakin_depth;
    //extern int breakin_node_limit;
    //ENGINE_EXTERN extern int level;               /* controls the strength of play */
    //ENGINE_EXTERN extern int min_level;           /* minimum level for adjustment schemes */
    //ENGINE_EXTERN extern int max_level;           /* minimum level for adjustment schemes */
    //extern int semeai_variations;   /* max variations considered reading semeai */
    //ENGINE_EXTERN extern float best_move_values[10];
    //ENGINE_EXTERN extern int best_moves[10];

    //extern int experimental_owl_ext;     /* use experimental owl (GAIN/LOSS) */
    //extern int experimental_semeai;      /* use experimental semeai module */
    //ENGINE_EXTERN extern int experimental_connections; /* use experimental connection module */
    //ENGINE_EXTERN extern int alternate_connections;    /* use alternate connection module */
    //ENGINE_EXTERN extern int owl_threats;              /* compute owl threats */
    //ENGINE_EXTERN extern int experimental_break_in;    /* use experimental module breakin.c */
    //ENGINE_EXTERN extern int cosmic_gnugo;             /* use center oriented influence */
    //ENGINE_EXTERN extern int large_scale;              /* seek large scale captures */

    //extern int thrashing_dragon;        /* Dead opponent's dragon trying to live */
    //extern char thrashing_stone[BOARDMAX];        /* All thrashing stones. */

    /* Experimental reading */
    //ENGINE_EXTERN extern char *rgoal;
    //ENGINE_EXTERN extern int goallib;



    /* Arrays pointing out the closest worms from each vertex.  The first
     * one is the closest worms of either color, the last two ones ignore
     * worms of the other color.  Beyond a certain distance from any worm
     * no close worm is listed at all.  Only the closest worm is listed
     * and if more than one are equally close they are all listed. The
     * number of equally close worms is given in the number_*_worms
     * arrays. If more than MAX_CLOSE_WORMS are equally close, none is
     * listed.
     *
     * See compute_effective_worm_sizes() in worm.c for details.
     */
#define MAX_CLOSE_WORMS 4
    //extern int close_worms[BOARDMAX][MAX_CLOSE_WORMS];
    //extern int number_close_worms[BOARDMAX];
    //extern int close_black_worms[BOARDMAX][MAX_CLOSE_WORMS];
    //extern int number_close_black_worms[BOARDMAX];
    //extern int close_white_worms[BOARDMAX][MAX_CLOSE_WORMS];
    //extern int number_close_white_worms[BOARDMAX];

    //ENGINE_EXTERN extern int false_eye_territory[BOARDMAX];
    //extern int forced_backfilling_moves[BOARDMAX];

    //ENGINE_EXTERN extern double slowest_time;      /* Timing statistics */
    //ENGINE_EXTERN extern int slowest_move;
    //ENGINE_EXTERN extern int slowest_movenum;
    //ENGINE_EXTERN extern double total_time;


    /* Store possible break-ins in initial position to generate move reasons
     * later.
     */
    struct break_in_data {
        int str;
        int move;
    };
    struct intrusion_data
    {
        int source_pos; 	/* Stone from which intrusion originates.*/
        int strength_pos;     /* Position of the intrusion influence soure. */
        float strength;
        float attenuation;
    };
    /* Maximum number of regions allowed between territory, moyo, and area.
     * FIXME: This number is vastly exaggerated. Should be possible to
     * come up with a much better upper bound.
     */ 
#define MAX_REGIONS (3*MAX_BOARD*MAX_BOARD + 1)

#define MAX_INTRUSIONS (2 * MAX_BOARD * MAX_BOARD)

    struct influence_data
    {
        char safe[BOARDMAX];

        float white_influence[BOARDMAX]; 	/* Accumulated influence. */
        float black_influence[BOARDMAX]; 	/* Accumulated influence. */
        float white_strength[BOARDMAX];  	/* Strength of influence source. */
        float black_strength[BOARDMAX];  	/* Strength of influence source. */
        float white_attenuation[BOARDMAX]; 
        float black_attenuation[BOARDMAX];
        float white_permeability[BOARDMAX];
        float black_permeability[BOARDMAX];

        int territory_segmentation[BOARDMAX];
        int moyo_segmentation[BOARDMAX];
        int area_segmentation[BOARDMAX];
        int region_type[MAX_REGIONS];
        int region_size[MAX_REGIONS];
        float region_territorial_value[MAX_REGIONS];
        int number_of_regions;

        int is_territorial_influence; /* 0 only if computing escape_influence.*/

        float territory_value[BOARDMAX];
        int non_territory[BOARDMAX];
        int captured;

        int color_to_move; /* Which color is in turn to move. */

        int queue[MAX_BOARD * MAX_BOARD];     /* Points receiving influence. */

        int intrusion_counter;
        struct intrusion_data intrusions[MAX_INTRUSIONS];

        int id;
    };


#define MAX_BREAK_INS 50

#define MAX_CACHE_DEPTH 	5
#define MAX_READING_CACHE_DEPTH 5
#define MAX_READING_CACHE_SIZE 100

#define MAX_OWL_CACHE_DEPTH 0
#define MAX_OWL_CACHE_SIZE 150

#define MAX_CONNECTION_CACHE_DEPTH 5
#define MAX_CONNECTION_CACHE_SIZE 100

#define MAX_BREAKIN_CACHE_DEPTH 1
#define MAX_BREAKIN_CACHE_SIZE 150


    /* We use the same data structure for all of the caches. Some of the entries
     * below are unused for some of the caches.
     */
    struct persistent_cache_entry {
        int boardsize;
        int movenum;
        char board[BOARDMAX];
        int stack[MAX_CACHE_DEPTH];
        int move_color[MAX_CACHE_DEPTH];
        enum routine_id routine;
        int apos; /* first input coordinate */
        int bpos; /* second input coordinate */
        int cpos; /* third input coordinate */
        Hash_data goal_hash; /* hash of the goal in break-in reading */
        int result;
        int result_certain;
        int remaining_depth;
        int node_limit; 
        int move; /* first result coordinate */
        int move2; /* second result coordinate */
        int cost; /* Usually no. of tactical nodes spent on this reading result. */
        int score; /* Heuristic guess of the worth of the cache entry. */
    };
    struct local_owl_data {
        char goal[BOARDMAX];
        char boundary[BOARDMAX];

        /* FIXME: neighbors[] and escape_values[] are never recomputed.
         *	    Consider moving these arrays from stack to a or
         *	    dynamic variable so it is not copied around in
         *	    do_push_owl().  Be aware of semeai code though.
         */
        char neighbors[BOARDMAX];

        char escape_values[BOARDMAX];
        int color;

        struct eye_data my_eye[BOARDMAX];
        /* array of half-eye data for use during owl reading */
        struct half_eye_data half_eye[BOARDMAX];

        int lunch[MAX_LUNCHES];
        int lunch_attack_code[MAX_LUNCHES];
        int lunch_attack_point[MAX_LUNCHES];
        int lunch_defend_code[MAX_LUNCHES];
        int lunch_defense_point[MAX_LUNCHES];
        char inessential[BOARDMAX];

        int lunches_are_current; /* If true, owl lunch data is current */  

        char safe_move_cache[BOARDMAX];

        /* This is used to organize the owl stack. */
        struct local_owl_data *restore_from;
    };

    /* Callback function that implements the computation of the active area.
     * This function has to be provided by each cache.
     */
    typedef void (*compute_active_area_fn)(struct persistent_cache_entry *entry,
            const char goal[BOARDMAX],
            int goal_color);
    ENGINE_EXTERN extern void compute_active_owl_area(struct persistent_cache_entry *entry,
            const char goal[BOARDMAX],
            int goal_color);
    ENGINE_EXTERN extern void compute_active_reading_area(struct persistent_cache_entry *entry,
            const char reading_shadow[BOARDMAX],
            int dummy);
    ENGINE_EXTERN extern void compute_active_connection_area(struct persistent_cache_entry *entry,
            const char
            connection_shadow[BOARDMAX],
            int goal_color);
    ENGINE_EXTERN extern void compute_active_breakin_area(struct persistent_cache_entry *entry,
            const char breakin_shadow[BOARDMAX],
            int dummy);

    struct persistent_cache {
        int max_size; /* Size of above array. */
        int max_stackp; /* Don't store positions with stackp > max_stackp. */
        float age_factor; /* Reduce value of old entries with this factor. */
        char *name; /* For debugging purposes. */
        compute_active_area_fn compute_active_area;
        struct persistent_cache_entry* table; /* Array of actual results. */
        int current_size; /* Current number of entries. */
        int last_purge_position_number;
    };
    /* Incremental string data. */
    struct string_data {
        int color;                       /* Color of string, BLACK or WHITE */
        int size;                        /* Number of stones in string. */
        int origin;                      /* Coordinates of "origin", i.e. */
        /* "upper left" stone. */
        int liberties;                   /* Number of liberties. */
        int libs[MAX_LIBERTIES];         /* Coordinates of liberties. */
        int neighbors;                   /* Number of neighbor strings */
        int neighborlist[MAXCHAIN];      /* List of neighbor string numbers. */
        int mark;                        /* General purpose mark. */
    };
    /* we keep the address and the old value */
    struct change_stack_entry {
        int *address;
        int value;
    };

    /* we keep the address and the old value */
    struct vertex_stack_entry {
        Intersection *address;
        int value;
    };

    /* Experimental results show that the average number of change stack
     * entries per move usually is in the 20-30 range and very seldom
     * exceeds 40. But since we have no way to recover from running out of
     * stack space, we allocate with a substantial safety margin.
     */
#define STACK_SIZE 80 * MAXSTACK

    typedef struct {
        int reason1;
        int what1;
        int reason2;
        int what2;
    } Reason_set;

    struct handicap_match {
        int value;
        int anchor;
        struct pattern *pattern;
        int ll;
    };
    typedef struct {

        /* clock parameters */
        int    clock_on;
        int    ready;
        double main_time;
        double byoyomi_time; /* zero if no byo-yomi */
        int    byoyomi_stones;

        /* clock status */
        double timer[3];
        double btimer[3];
        int    byoyomi[3];
        int    dead[3];

        /* dates of each move */
        int    moveno; /* invariant: COLOR(clk.moveno) = color of last move */
#define CLOCK_MAX_MOVES           500   /* max number of moves for a game */

        double date[CLOCK_MAX_MOVES];

        /* adapative system parameters */
        int    autolevel_on;
        double level;  /* FIXME: Why is this a double and not an int? */
        double levels[CLOCK_MAX_MOVES];
        double expected[CLOCK_MAX_MOVES];
        double error; /* time/move estimation error */
    } gnugo_clock;


    struct globals
    {
        struct pattern_db read_defend_db;
        struct pattern_db read_attack_db;

        /* The transposition table itself. */
        Transposition_table ttable;

        /* Array for use by TRANSFORM() macro. */
        int transformation[MAX_OFFSET][8];

        /* Spiral orders for DFA matching and building. */
        int spiral[DFA_MAX_ORDER][8];

        SGFTree *aftermath_sgftree;

        struct aftermath_data aftermath;

        int play_aftermath_current_board[BOARDMAX];
        int play_aftermath_current_color ;

        int stones_on_board_stone_count_for_position;
        int stones_on_board_white_stones;
        int stones_on_board_black_stones;

        int keyhash_init_is_initialized;

        gnugo_clock clk;
        int aa_status[BOARDMAX]; /* ALIVE, DEAD or CRITICAL */
        int forbidden[BOARDMAX];
        int aa_values[BOARDMAX];

        /* FIXME: Move these to a struct and pass to callback through the
         * *data parameter.
         */
        int current_minsize;
        struct aa_move *current_attacks;
        int conditional_attack_point[BOARDMAX];

        int dragon_escape_mx[BOARDMAX];
        int dragon_escape_mx_initialized;
        int new_dragon_origins[BOARDMAX];

        int filllib_adjs[MAXCHAIN];
        int filllib_libs[MAXLIBS];

        /* Global variables remembering which symmetries the position has. */
        int horizontally_symmetric; /* symmetry with respect to K column */
        int vertically_symmetric;   /* symmetry with respect to 10 row */
        int diagonally_symmetric;   /* with respect to diagonal from UR to LL */

        /* Storage for values collected during pattern matching. */
        int fuseki_moves[MAX_BOARD * MAX_BOARD];
        int fuseki_value[MAX_BOARD * MAX_BOARD];
        int num_fuseki_moves;
        int fuseki_total_value;

        int search_mask[BOARDMAX];

        /*
         * Sets up free handicap placement stones, returning the number of
         * placed handicap stones.
         */

        int remaining_handicap_stones;
        int total_handicap_stones;

#define MAX_HANDICAP_MATCHES 40

        struct handicap_match handicap_matches[MAX_HANDICAP_MATCHES];
        int number_of_matches;

        int hash_is_initialized;


        /* Random values for the board hash function. For stones and ko position. */
        Hash_data hash_white_hash[BOARDMAX];
        Hash_data hash_black_hash[BOARDMAX];
        Hash_data hash_ko_hash[BOARDMAX];

#define HASHVALUE_NUM_DIGITS (1 + (CHAR_BIT * SIZEOF_HASHVALUE - 1) / 4)
#define BUFFER_SIZE (1 + NUM_HASHVALUES * HASHVALUE_NUM_DIGITS)

        char hashdata_to_string_buffer[BUFFER_SIZE];

        float accumulate_influence_working[BOARDMAX];
        int accumulate_influence_working_area_initialized;

        char segment_region_marked[BOARDMAX];

        int compute_escape_influence_cached_board[BOARDMAX];
        char compute_escape_influence_escape_values[BOARDMAX][2];
        int compute_escape_influence_active_caches[2] ;

        int PROFILE_MATCHER_totals[6];

        unsigned int class_mask[NUM_DRAGON_STATUS][3];

        /* Stone locations are stored in this array. They might be needed by callback
         * function.
         */
        /* Data. */
        int dfa_board_size;
        int dfa_p[DFA_BASE * DFA_BASE];

        int matchpat_num_stones[2*BOARDMAX];
        int matchpat_pattern_stones[BOARDMAX];

        char eyevalue_to_string_result[30];

        Hash_data remembered_board_hashes[MAXSTACK];

        struct local_owl_data *owl_defend_owl;

        int location_to_string_init;
        char location_to_string_buf[BOARDSIZE][5];

        int nodes_connect;

        /* Used by alternate connections. */
        char connection_shadow[BOARDMAX];

        char breakin_shadow[BOARDMAX];


        /* Statistics. */
        int global_connection_node_counter;

        /* Statistics. */
        int reading_node_counter;
        int nodes_when_called;

        /* We would like to initialize liberty_mark to -1, but some
         * compilers warn, quite correctly, that -1 is not an unsigned
         * number.
         */
        unsigned fast_defense_liberty_mark;
        unsigned fast_defense_lm[BOARDMAX];

        int defend_lib_score[6]               ;
        int defend_not_adjacent_lib_score[5]  ;
        int defend_capture_score[6]          ;
        int defend_atari_score[6]            ;
        int defend_save_score[6]             ;
        int defend_open_score[5]             ;
        int attack_own_lib_score[5]          ;
        int attack_string_lib_score[6]       ;
        int attack_capture_score[6]          ;
        int attack_save_score[6]             ;
        int attack_open_score[5]             ;
        int defend_not_edge_score            ;
        int attack_not_edge_score            ;
        int attack_ko_score                  ;
        int cannot_defend_penalty            ;
        int safe_atari_score                 ;

        SGFTree local_tree;
        int safe_move_initialized;

        int safe_move_cache[BOARDMAX][2];
        int safe_move_cache_when[BOARDMAX][2];

        /* Private variable remembering the random seed. */
        unsigned int random_seed;
        int warned;
        /* Internal timers for assessing time spent on various tasks. */
#define NUMBER_OF_TIMERS 4
        double timers[NUMBER_OF_TIMERS];
        /* Globals */
        int gg;      /* stores the gravity center of the goal */
        /* Element at origin of each worm stores allocated worm number. */
        unsigned char dragon_num[BOARDMAX];

        int next_white;		/* next worm number to allocate */
        int next_black;

        int sgf_column;

        FILE *sgffile;
        char *sgferr;
        int sgferrarg;
        int sgferrpos;
        int lookahead;
        int find_worm_threats_libs[MAXLIBS];
        int find_worm_attacks_and_defenses_libs[MAXLIBS];
        int do_compute_effective_worm_sizes_worms[BOARDMAX][2*(MAX_BOARD-1)];


        /* Point redistribution */
        int replacement_map[BOARDMAX];

        /* The color for which we are evaluating moves. */
        int current_color;

        int next_reason;

        /* Connections */
        int conn_worm1[MAX_CONNECTIONS];
        int conn_worm2[MAX_CONNECTIONS];
        int next_connection;



        char *gg_optarg;
        int gg_optind; // = 1;
        int __getopt_initialized;
        char *nextchar;
        int gg_opterr; // = 1;
        int gg_optopt; // = '?';
        int ordering;
        char *posixly_correct;
        int dragon2_initialized;
        int lively_white_dragons;
        int lively_black_dragons;
        /* Position numbers for which various examinations were last made. */
        int worms_examined;
        int initial_influence_examined;
        int dragons_examined_without_owl;
        int dragons_examined;
        int initial_influence2_examined;
        int dragons_refinedly_examined;
        /* These are used during the calculations of eye spaces. */
        int black_domain[BOARDMAX];
        int white_domain[BOARDMAX];
        /* Semeai worms are worms whose capture wins the semeai. */
        int matches_found;
        char found_matches[BOARDMAX];

        struct local_owl_data *owl_stack[2 * MAXSTACK];
        int owl_stack_size;
        int owl_stack_pointer;

        int dragon_goal_worms[MAX_DRAGONS][MAX_GOAL_WORMS];
        char *rgoal;
        int goallib;

#define MAX_SEMEAI_WORMS 20
        int s_worms;
        int semeai_worms[MAX_SEMEAI_WORMS];
        int important_semeai_worms[MAX_SEMEAI_WORMS];

        /* Whether one color prefers to get a ko over a seki. */
        int prefer_ko;

        /* Usually it's a bad idea to include the opponent worms involved in
         * the semeai in the eyespace. For some purposes (determining a
         * definite lack of eyespace, finding certain vital moves), however,
         * we want to do that anyway. Then set this variable to 1 before
         * calling owl_estimate_life() and reset it afterwards.
         *
         * FIXME: We should implement a nicer mechanism to propagate this
         *        information to owl_lively(), where it's used.
         */
        int include_semeai_worms_in_eyespace;


        int result_certain;

        /* Statistics. */
        int local_owl_node_counter;
        /* Node limitation. */
        int global_owl_node_counter;

        struct local_owl_data *current_owl_data;
        struct local_owl_data *other_owl_data;

        int goal_worms_computed;
        int owl_goal_worm[MAX_GOAL_WORMS];

        /* Used internally by mapping functions. */
        int map_size;
        char used_index[MAXEYE];

        unsigned int x[25];
        int k;
        int rand_initialized; // = 0;
        int first_nonopt;
        int last_nonopt;
        int level;

        int          board_size;             /* board size (usually 19) */
        Intersection board[BOARDSIZE];       /* go board */
        int          board_ko_pos;
        int          black_captured;   /* num. of black stones captured */
        int          white_captured;

        Intersection initial_board[BOARDSIZE];
        int          initial_board_ko_pos;
        int          initial_white_captured;
        int          initial_black_captured;
        int          move_history_color[MAX_MOVE_HISTORY];
        int          move_history_pos[MAX_MOVE_HISTORY];
        int          move_history_pointer;

        float        komi;
        int          movenum;      /* movenumber - used for debug output */

        char         shadow[BOARDMAX];      /* reading tree shadow */

        int chinese_rules;
        int allow_suicide;
        int next_worm_list[BOARDMAX];

        int stackp;                /* stack pointer */
        int count_variations;      /* count (decidestring) */
        SGFTree *sgf_dumptree;
        int position_number;

        /* Coordinates for the eight directions, ordered
         * south, west, north, east, southwest, northwest, northeast, southeast.
         * Defined in board.c.
         */
        int deltai[8]; /* = { 1,  0, -1,  0,  1, -1, -1, 1}; */
        int deltaj[8]; /* = { 0, -1,  0,  1, -1, -1,  1, 1}; */
        int delta[8];  /* = { NS, -1, -NS, 1, NS-1, -NS-1, -NS+1, NS+1}; */

        struct stats_data stats;

        /* Miscellaneous debug options. */
        int quiet;		/* Minimal output. */
        int verbose;		/* Bore the opponent. */
        int allpats;		/* generate all patterns, even small ones */
        int printworms;		/* print full data on each string */
        int printmoyo;		/* print moyo board each move */
        int printdragons;	/* print full data on each dragon */
        int printboard;		/* print board each move */
        int showstatistics;	/* print statistics */
        int profile_patterns;	/* print statistics of pattern usage */
        char outfilename[128];  /* output file (-o option) */
        int output_flags;       /* amount of output to outfile */



        int debug;		/* debug flags */
        int hashflags;		/* hash flags */
        int fusekidb;            /* use fuseki database */
        int disable_fuseki;      /* do not generate fuseki moves */
        int josekidb;            /* use joseki database */
        //extern int level;		/* controls depth of reading */
        int semeai_variations;   /* max variations considered reading semeai */
        int showtime;		/* print genmove time */
        int showscore;		/* print score */
        //int chinese_rules;       /* use chinese (area) rules for counting */
        int experimental_owl_ext;     /* use experimental owl (GAIN/LOSS) */
        int experimental_connections; /* use experimental connection module */
        int alternate_connections;    /* use alternate connection module */
        int owl_threats;              /* compute owl threats */
        int capture_all_dead;         /* capture all dead opponent stones */
        int play_out_aftermath; /* make everything unconditionally settled */
        int resign_allowed;           /* allows GG to resign hopeless games */
        int play_mirror_go;           /* try to play mirror go if possible */
        int mirror_stones_limit;      /* but stop at this number of stones */
        int gtp_version;              /* version of Go Text Protocol */

#if EXPERIMENTAL_READING
        int defend_by_pattern;  /* use patterns for tactical reading defense */
        int attack_by_pattern;  /* use patterns for tactical reading attack */
#endif

        /* Mandatory values of reading parameters. Normally -1, if set at 
         * these override the values derived from the level. */
        int mandated_depth;
        int mandated_backfill_depth;
        int mandated_backfill2_depth;
        int mandated_break_chain_depth;
        int mandated_superstring_depth;
        int mandated_fourlib_depth;
        int mandated_ko_depth;
        int mandated_branch_depth;
        int mandated_aa_depth;
        int mandated_owl_distrust_depth;
        int mandated_owl_branch_depth;
        int mandated_owl_reading_depth;
        int mandated_owl_node_limit; 

        /* Keep this as 2D until we change the entire API. */
        float potential_moves[MAX_BOARD][MAX_BOARD];

        int limit_search;  /* limit move search to a portion of the board */
        int oracle_exists; /* oracle is available for consultation        */
        int metamachine;   /* use metamachine_genmove                     */


        char *__getopt_nonoption_flags;

        int nonoption_flags_max_len;
        int nonoption_flags_len;

        int original_argc;
        char *const *original_argv;

        struct pattern_db aa_attackpat_db;

        /* 
         * Define all global variables used within the engine.
         */

        int thrashing_dragon; /* Dead opponent's dragon trying to live. */
        char thrashing_stone[BOARDMAX]; /* All thrashing stones. */

        //int hashflags;

        //float potential_moves[MAX_BOARD][MAX_BOARD];

        /* Used by reading. */
        int depth;              /* deep reading cut off */
        int backfill_depth;     /* deep reading cut off */
        int backfill2_depth;    /* deep reading cut off */
        int break_chain_depth;  /* deep reading cut off */
        int superstring_depth;  /* deep reading cut off */
        int fourlib_depth;      /* deep reading cut off */
        int ko_depth;           /* deep reading cut off */
        int branch_depth;       /* deep reading cut off */
        int aa_depth;
        int depth_offset;       /* keeps track of temporary depth changes */
        int owl_distrust_depth;   /* below this owl trusts the optics code */
        int owl_branch_depth;     /* below this owl tries only one variation */
        int owl_reading_depth;    /* owl does not read below this depth */
        int owl_node_limit;       /* maximum number of nodes considered */
        int semeai_branch_depth;
        int semeai_branch_depth2;
        int semeai_node_limit;
        int connect_depth;	/* Used by Tristan Cazenave's connection reader. */
        int connect_depth2;     /* Used by alternater connection reader. */
        int connection_node_limit; 
        int breakin_node_limit; /* Reading limits for break_in/block_off reading */
        int breakin_depth;
        //int mandated_depth;             /* deep reading cut off, mandated value */
        //int mandated_backfill_depth;    /* deep reading cut off, mandated value */
        //int mandated_backfill2_depth;   /* deep reading cut off, mandated value */
        //int mandated_break_chain_depth; /* deep reading cut off, mandated value */
        //int mandated_superstring_depth; /* deep reading cut off, mandated value */
        //int mandated_fourlib_depth;     /* deep reading cut off, mandated value */
        //int mandated_ko_depth;          /* deep reading cut off, mandated value */
        //int mandated_branch_depth;      /* deep reading cut off, mandated value */
        //int mandated_aa_depth;
        //int mandated_owl_distrust_depth;  
        //int mandated_owl_branch_depth;  
        //int mandated_owl_reading_depth; 
        //int mandated_owl_node_limit;    


        /* Miscellaneous. */
        //int quiet             ;  /* minimal output */
        //int showstatistics    ;  /* print statistics */
        //int profile_patterns  ;  /* print statistics of pattern usage */
        //int allpats           ;  /* generate all patterns, even small ones */
        //int printworms        ;  /* print full data on each string */
        //int printmoyo         ;  /* print moyo board each move */
        //int printboard        ;  /* print board each move */
        int loading           ;  /* TRUE if last loaded move comes from file */
        //int fusekidb          ;  /* use fuseki database */
        //int disable_fuseki    ;  /* do not generate fuseki moves */
        //int josekidb          ;  /* use fuseki database */
        //int showtime          ;  /* print time to find move */
        //int showscore         ;  /* print estimated score */
        //int level             ; /* strength; up to 10 supported */
        int min_level         ;
        int max_level         ;
        //int debug             ;  /* controls debug output */
        //int verbose           ;  /* trace level */
        //char outfilename[128] ; /* output file (-o option) */
        //int output_flags      ; /* amount of output to outfile */
        //int limit_search      ;  /* limit search to a portion of the board */
        //int metamachine       ;  /* use metamachine_genmove */
        //int oracle_exists     ;  /* oracle is available for consultation   */

        int disable_threat_computation ;
        int disable_endgame_patterns   ;
        int doing_scoring              ;

        //int chinese_rules       ;    /* ruleset choice for GMP connection */
        /* use experimental connection module */
        //int experimental_connections;
        /* use alternate connection reading algorithm */
        //int alternate_connections;
        /* compute owl threats */
        //int owl_threats; 
        /* use experimental owl extension (GAIN/LOSS) */
        //int experimental_owl_ext;
        /* use experimental territory break-in module */
        int experimental_break_in;
        /* use central oriented influence */
        int cosmic_gnugo;

        //int capture_all_dead    ;    /* capture all dead opponent stones */
        //int play_out_aftermath  ;    /* make everything unconditionally settled */
        //int resign_allowed      ;    /* resign hopeless games */
        int large_scale         ;    /* search for large scale owl moves */

        //int play_mirror_go      ;    /* try to play mirror go if possible */
        //int mirror_stones_limit ;   /* but stop at this number of stones */

        //int gtp_version         ;    /* Use GTP version 2 by default. */

        float best_move_values[10];
        int   best_moves[10];

        int close_worms[BOARDMAX][4];
        int number_close_worms[BOARDMAX];
        int close_black_worms[BOARDMAX][4];
        int number_close_black_worms[BOARDMAX];
        int close_white_worms[BOARDMAX][4];
        int number_close_white_worms[BOARDMAX];

        int false_eye_territory[BOARDMAX];
        int forced_backfilling_moves[BOARDMAX];

        struct worm_data      worm[BOARDMAX];
        struct dragon_data    dragon[BOARDMAX];
        int                   number_of_dragons;
        struct dragon_data2   *dragon2;
        struct half_eye_data  half_eye[BOARDMAX];
        struct eye_data       black_eye[BOARDMAX];
        struct eye_data       white_eye[BOARDMAX];
        struct vital_eye_points black_vital_points[BOARDMAX];
        struct vital_eye_points white_vital_points[BOARDMAX];
        struct surround_data  surroundings[MAX_SURROUND];
        int                   surround_pointer;

        int cutting_points[BOARDMAX];

        double slowest_time ;
        int    slowest_move ;
        int    slowest_movenum ;
        double total_time;


        int use_optimistic_territory;
        float minimum_value_weight  ;
        float maximum_value_weight  ;
        float invasion_malus_weight ;
        float territorial_weight    ;
        float strategical_weight    ;
        float attack_dragon_weight  ;
        float followup_weight       ;


        struct board_cache_entry accuratelib_cache[BOARDMAX][2];
        /* approxlib() cache. */
        struct board_cache_entry approxlib_cache[BOARDMAX][2];



        /* All these data structures are declared in move_reasons.h */

        struct move_data move[BOARDMAX];
        struct move_reason move_reasons[MAX_MOVE_REASONS];

        /* Unordered sets (currently pairs) of move reasons / targets */
        Reason_set either_data[MAX_EITHER];
        int next_either;
        Reason_set all_data[MAX_ALL];
        int next_all;

        /* Eye shapes */
        int eyes[MAX_EYES];
        int eyecolor[MAX_EYES];
        int next_eye;

        /* Lunches */
        int lunch_dragon[MAX_LUNCHES]; /* eater */
        int lunch_worm[MAX_LUNCHES];   /* food */
        int next_lunch;



        /* Attack threats that are known to be sente locally. */
        int known_good_attack_threats[BOARDMAX][MAX_ATTACK_THREATS];

        /* The board state itself. */
        //int          board_size ; /* board size */
        //Intersection board[BOARDSIZE];
        //int          board_ko_pos;
        //int          white_captured;    /* number of black and white stones captured */
        //int          black_captured;

        //Intersection initial_board[BOARDSIZE];
        //int          initial_board_ko_pos;
        //int          initial_white_captured;
        //int          initial_black_captured;
        //int          move_history_color[MAX_MOVE_HISTORY];
        //int          move_history_pos[MAX_MOVE_HISTORY];
        //int          move_history_pointer;

        //float komi;
        //int movenum;
        //int allow_suicide ;  /* allow opponent to make suicide moves */


        //char shadow[BOARDMAX];

        /* Hashing of positions. */
        Hash_data board_hash;

        //int stackp;             /* stack pointer */
        //int position_number;    /* position number */

        /* Some statistics gathered partly in board.c and hash.c */
        //struct stats_data stats;

        /* Variation tracking in SGF trees: */
        //int count_variations  ;  /* used by decide_string */
        int sgf_dump          ;  /* used by decide_string */
        //SGFTree *sgf_dumptree ;

        /* When to use pattern-base reading */
        int defend_by_pattern ;
        int attack_by_pattern ;

        struct pattern_db attpat_db;
        struct pattern_db influencepat_db;
        struct pattern_db barrierspat_db;
        struct break_in_data break_in_list[MAX_BREAK_INS];
        int num_break_ins;

        struct persistent_cache reading_cache;

        struct persistent_cache connection_cache;

        struct persistent_cache breakin_cache;

        struct persistent_cache owl_cache;

        struct pattern_db conn_db;

        /* cosmic_importance is a number between 0.0 and 1.0 ;
         * when cosmic_importance is 0.0, the default influence
         * values are used; when cosmic_importance is 1.0, GNU Go
         * will try to play an influence-oriented fuseki by 
         * over-estimatingthe potential territory values of moyos.
         * In the current implementation, cosmic_importance decreases 
         * slowly for 19*19 games from 1.0 at move 4 to 0.0 at move 120.
         */
        float cosmic_importance;


        struct pattern_db defpat_db;
        /* ================================================================ */
        /*                      data structures                      */
        /* ================================================================ */


        /* Main array of string information. */
        struct string_data string[MAX_STRINGS];

        /* Stacks and stack pointers. */
        struct change_stack_entry change_stack[STACK_SIZE];
        struct change_stack_entry *change_stack_pointer;

        struct vertex_stack_entry vertex_stack[STACK_SIZE];
        struct vertex_stack_entry *vertex_stack_pointer;


        /* Index into list of strings. The index is only valid if there is a
         * stone at the vertex.
         */
        int string_number[BOARDMAX];


        /* The stones in a string are linked together in a cyclic list. 
         * These are the coordinates to the next stone in the string.
         */
        int next_stone[BOARDMAX];

        /* Number of the next free string. */
        int next_string;


        /* For marking purposes. */
        int ml[BOARDMAX];
        int liberty_mark;
        int string_mark;

        /* Statistics. */
        int trymove_counter;

        /* Coordinates for the eight directions, ordered
         * south, west, north, east, southwest, northwest, northeast, southeast.
         */
        //int deltai[8] ;
        //int deltaj[8];
        //int delta[8];


        /* Influence computed for the initial position, i.e. before making
         * some move.
         */
        struct influence_data initial_black_influence;
        struct influence_data initial_white_influence;

        /* Influence computed after some move has been made. */
        struct influence_data move_influence;
        struct influence_data followup_influence;

        /* Influence used for estimation of escape potential. */
        struct influence_data escape_influence;

        /* Pointer to influence data used during pattern matching. */
        struct influence_data *current_influence;


        /* Thresholds values used in the whose_moyo() functions */
        struct moyo_determination_data moyo_data;
        struct moyo_determination_data moyo_restricted_data;

        /* Thresholds value used in the whose_territory() function */
        float territory_determination_value; 



        /* This curve determines how much influence is needed at least to claim
         * an intersection as territory, in dependence of the "center value".
         * (In the center, more effort is needed to get territory!)
         * The center value is at the moment defined as follows:
         * If d1, d2 are the distance to vertical and horizontal border, resp.,
         * with d1<d2, then
         * central = 3 * d1 + min(d2, 4)
         * So this is mainly a function of the distance to the border; the
         * distance to the second-nearest border gives a small correction of at
         * most 4. This distinguishes edge and corner positions.
         *
         * The values for intersections close to a corner or to the edge have
         * to be consistent such that standard corner enclosure etc. are
         * sufficient to claim territory. The center values are more arbitrary
         * suspect to tuning.
         */

        struct interpolation_data min_infl_for_territory;

        /* Determines the territory correction factor in dependence of the ratio
         * ( influence of stronger color / min_infl_for_territory(intersection))
         */
        struct interpolation_data territory_correction;





        /* If set, print influence map when computing this move. Purely for
         * debugging.
         */
        int debug_influence;

        /* Assigns an id to all influence computations for reference in the
         * delta territory cache.
         */
        int influence_id;


        /* Cache of delta_territory_values. */
        float delta_territory_cache[BOARDMAX];
        float followup_territory_cache[BOARDMAX];
        Hash_data delta_territory_cache_hash[BOARDMAX];
        int territory_cache_position_number;
        int territory_cache_influence_id;
        int territory_cache_color;

        struct pattern_db endpat_db;


        FILE *to_gnugo_stream, *from_gnugo_stream;
        char gnugo_line[128];
        int gnugo_line_length;

        int pfd_a[2];
        int pfd_b[2];
        /* Stack of trial moves to get to current
         * position and which color made them. Perhaps 
         * this should be one array of a structure 
         */
        int stack[MAXSTACK];
        int move_color[MAXSTACK];
        Hash_data board_hash_stack[MAXSTACK];

        int komaster;
        int kom_pos;
        /* Arrays with random numbers for Zobrist hashing of input data (other
         * than the board position). If you add an array here, do not forget
         * to also initialize it in keyhash_init() below.
         */
        Hash_data komaster_hash[NUM_KOMASTER_STATES];
        Hash_data kom_pos_hash[BOARDMAX];
        Hash_data target1_hash[BOARDMAX];
        Hash_data target2_hash[BOARDMAX];
        Hash_data routine_hash[NUM_CACHE_ROUTINES];

        struct pattern *aa_attackpat;
        struct pattern *attpat;
        struct pattern *barrierspat;
        struct pattern *conn;
        struct pattern *defpat;
        struct pattern *endpat;
        struct pattern *influencepat;
        struct pattern *read_defend;
        struct pattern *read_attack;

    };

#define global_aa_attackpat (global()->aa_attackpat)
#define global_attpat (global()->attpat)
#define global_barrierspat (global()->barrierspat)
#define global_conn (global()->conn)
#define global_defpat (global()->defpat)
#define global_endpat (global()->endpat)
#define global_read_defend (global()->read_defend)
#define global_read_attack (global()->read_attack)
#define global_influencepat (global()->influencepat)


#define global___getopt_initialized (global()->__getopt_initialized)
#define global___getopt_nonoption_flags (global()->___getopt_nonoption_flags)
#define global_aa_attackpat_db (global()->aa_attackpat_db)
#define global_aa_depth (global()->aa_depth)
#define global_accuratelib_cache (global()->accuratelib_cache)
#define global_all_data (global()->all_data)
#define global_allow_suicide (global()->allow_suicide)
#define global_allpats (global()->allpats)
#define global_alternate_connections (global()->alternate_connections)
#define global_approxlib_cache (global()->approxlib_cache)
#define global_attack_by_pattern (global()->attack_by_pattern)
#define global_attack_dragon_weight (global()->attack_dragon_weight)
#define global_attpat_db (global()->attpat_db)
#define global_backfill_depth (global()->backfill_depth)
#define global_backfill2_depth (global()->backfill2_depth)
#define global_barrierspat_db (global()->barrierspat_db)
#define global_best_move_values (global()->best_move_values)
#define global_best_moves (global()->best_moves)
#define global_black_captured (global()->black_captured)
#define global_black_domain (global()->black_domain)
#define global_black_eye (global()->black_eye)
#define global_black_vital_points (global()->black_vital_points)
#define global_board (global()->board)
#define global_board_hash (global()->board_hash)
#define global_board_hash_stack (global()->board_hash_stack)
#define global_board_ko_pos (global()->board_ko_pos)
#define global_board_size (global()->board_size)
#define global_branch_depth (global()->branch_depth)
#define global_break_chain_depth (global()->break_chain_depth)
#define global_break_in_list (global()->break_in_list)
#define global_breakin_cache (global()->breakin_cache)
#define global_breakin_depth (global()->breakin_depth)
#define global_breakin_node_limit (global()->breakin_node_limit)
#define global_capture_all_dead (global()->capture_all_dead)
#define global_change_stack (global()->change_stack)
#define global_change_stack_pointer (global()->change_stack_pointer)
#define global_chinese_rules (global()->chinese_rules)
#define global_close_black_worms (global()->close_black_worms)
#define global_close_white_worms (global()->close_white_worms)
#define global_close_worms (global()->close_worms)
#define global_conn_db (global()->conn_db)
#define global_conn_worm1 (global()->conn_worm1)
#define global_conn_worm2 (global()->conn_worm2)
#define global_connect_depth (global()->connect_depth)
#define global_connect_depth2 (global()->connect_depth2)
#define global_connection_cache (global()->connection_cache)
#define global_connection_node_limit (global()->connection_node_limit)
#define global_cosmic_gnugo (global()->cosmic_gnugo)
#define global_cosmic_importance (global()->cosmic_importance)
#define global_count_variations (global()->count_variations)
#define global_current_color (global()->current_color)
#define global_current_influence (global()->current_influence)
#define global_current_owl_data (global()->current_owl_data)
#define global_cutting_points (global()->cutting_points)
#define global_debug (global()->debug)
#define global_debug_influence (global()->debug_influence)
#define global_defend_by_pattern (global()->defend_by_pattern)
#define global_defpat_db (global()->defpat_db)
#define global_delta (global()->delta)
#define global_delta_territory_cache (global()->delta_territory_cache)
#define global_delta_territory_cache_hash (global()->delta_territory_cache_hash)
#define global_deltai (global()->deltai)
#define global_deltaj (global()->deltaj)
#define global_depth (global()->depth)
#define global_depth_offset (global()->depth_offset)
#define global_disable_endgame_patterns (global()->disable_endgame_patterns)
#define global_disable_fuseki (global()->disable_fuseki)
#define global_disable_threat_computation (global()->disable_threat_computation)
#define global_doing_scoring (global()->doing_scoring)
#define global_dragon (global()->dragon)
#define global_dragon_goal_worms (global()->dragon_goal_worms)
#define global_dragon2 (global()->dragon2)
#define global_dragon2_initialized (global()->dragon2_initialized)
#define global_dragons_examined (global()->dragons_examined)
#define global_dragons_examined_without_owl (global()->dragons_examined_without_owl)
#define global_dragons_refinedly_examined (global()->dragons_refinedly_examined)
#define global_either_data (global()->either_data)
#define global_endpat_db (global()->endpat_db)
#define global_escape_influence (global()->escape_influence)
#define global_experimental_break_in (global()->experimental_break_in)
#define global_experimental_connections (global()->experimental_connections)
#define global_experimental_owl_ext (global()->experimental_owl_ext)
#define global_eyecolor (global()->eyecolor)
#define global_eyes (global()->eyes)
#define global_false_eye_territory (global()->false_eye_territory)
#define global_first_nonopt (global()->first_nonopt)
#define global_followup_influence (global()->followup_influence)
#define global_followup_territory_cache (global()->followup_territory_cache)
#define global_followup_weight (global()->followup_weight)
#define global_forced_backfilling_moves (global()->forced_backfilling_moves)
#define global_found_matches (global()->found_matches)
#define global_fourlib_depth (global()->fourlib_depth)
#define global_from_gnugo_stream (global()->from_gnugo_stream)
#define global_fusekidb (global()->fusekidb)
#define global_gg_optarg (global()->gg_optarg)
#define global_gg_opterr (global()->gg_opterr)
#define global_gg_optind (global()->gg_optind)
#define global_gg_optopt (global()->gg_optopt)
#define global_global_owl_node_counter (global()->global_owl_node_counter)
#define global_gnugo_line (global()->gnugo_line)
#define global_gnugo_line_length (global()->gnugo_line_length)
#define global_goal_worms_computed (global()->goal_worms_computed)
#define global_goallib (global()->goallib)
#define global_gtp_version (global()->gtp_version)
#define global_half_eye (global()->half_eye)
#define global_hashflags (global()->hashflags)
#define global_important_semeai_worms (global()->important_semeai_worms)
#define global_include_semeai_worms_in_eyespace (global()->include_semeai_worms_in_eyespace)
#define global_influence_id (global()->influence_id)
#define global_initial_black_captured (global()->initial_black_captured)
#define global_initial_black_influence (global()->initial_black_influence)
#define global_initial_board (global()->initial_board)
#define global_initial_board_ko_pos (global()->initial_board_ko_pos)
#define global_initial_influence_examined (global()->initial_influence_examined)
#define global_initial_influence2_examined (global()->initial_influence2_examined)
#define global_initial_white_captured (global()->initial_white_captured)
#define global_initial_white_influence (global()->initial_white_influence)
#define global_Intersection initial_board (global()->Intersection initial_board)
#define global_invasion_malus_weight (global()->invasion_malus_weight)
#define global_josekidb (global()->josekidb)
#define global_k (global()->k)
#define global_known_good_attack_threats (global()->known_good_attack_threats)
#define global_ko_depth (global()->ko_depth)
#define global_kom_pos (global()->kom_pos)
#define global_kom_pos_hash (global()->kom_pos_hash)
#define global_komaster (global()->komaster)
#define global_komaster_hash (global()->komaster_hash)
#define global_komi (global()->komi)
#define global_large_scale (global()->large_scale)
#define global_last_nonopt (global()->last_nonopt)
#define global_level (global()->level)
#define global_liberty_mark (global()->liberty_mark)
#define global_limit_search (global()->limit_search)
#define global_lively_black_dragons (global()->lively_black_dragons)
#define global_lively_white_dragons (global()->lively_white_dragons)
#define global_loading (global()->loading)
#define global_local_owl_node_counter (global()->local_owl_node_counter)
#define global_lunch_dragon (global()->lunch_dragon)
#define global_lunch_worm (global()->lunch_worm)
#define global_mandated_aa_depth (global()->mandated_aa_depth)
#define global_mandated_backfill_depth (global()->mandated_backfill_depth)
#define global_mandated_backfill2_depth (global()->mandated_backfill2_depth)
#define global_mandated_branch_depth (global()->mandated_branch_depth)
#define global_mandated_break_chain_depth (global()->mandated_break_chain_depth)
#define global_mandated_depth (global()->mandated_depth)
#define global_mandated_fourlib_depth (global()->mandated_fourlib_depth)
#define global_mandated_ko_depth (global()->mandated_ko_depth)
#define global_mandated_owl_branch_depth (global()->mandated_owl_branch_depth)
#define global_mandated_owl_distrust_depth (global()->mandated_owl_distrust_depth)
#define global_mandated_owl_node_limit (global()->mandated_owl_node_limit)
#define global_mandated_owl_reading_depth (global()->mandated_owl_reading_depth)
#define global_mandated_superstring_depth (global()->mandated_superstring_depth)
#define global_map_size (global()->map_size)
#define global_matches_found (global()->matches_found)
#define global_max_level (global()->max_level)
#define global_maximum_value_weight (global()->maximum_value_weight)
#define global_metamachine (global()->metamachine)
#define global_min_infl_for_territory (global()->min_infl_for_territory)
#define global_min_level (global()->min_level)
#define global_minimum_value_weight (global()->minimum_value_weight)
#define global_mirror_stones_limit (global()->mirror_stones_limit)
#define global_ml (global()->ml)
#define global_move (global()->move)
#define global_move_color (global()->move_color)
#define global_move_history_color (global()->move_history_color)
#define global_move_history_pointer (global()->move_history_pointer)
#define global_move_history_pos (global()->move_history_pos)
#define global_move_influence (global()->move_influence)
#define global_move_reasons (global()->move_reasons)
#define global_movenum (global()->movenum)
#define global_movenum	 (global()->movenum)
#define global_moyo_data (global()->moyo_data)
#define global_moyo_restricted_data (global()->moyo_restricted_data)
#define global_next_all (global()->next_all)
#define global_next_connection (global()->next_connection)
#define global_next_either (global()->next_either)
#define global_next_eye (global()->next_eye)
#define global_next_lunch (global()->next_lunch)
#define global_next_reason (global()->next_reason)
#define global_next_stone (global()->next_stone)
#define global_next_string (global()->next_string)
#define global_next_worm_list (global()->next_worm_list)
#define global_nextchar (global()->nextchar)
#define global_nonoption_flags_len (global()->nonoption_flags_len)
#define global_nonoption_flags_max_len (global()->nonoption_flags_max_len)
#define global_num_break_ins (global()->num_break_ins)
#define global_number_close_black_worms (global()->number_close_black_worms)
#define global_number_close_white_worms (global()->number_close_white_worms)
#define global_number_close_worms (global()->number_close_worms)
#define global_number_of_dragons (global()->number_of_dragons)
#define global_oracle_exists (global()->oracle_exists)
#define global_ordering (global()->ordering)
#define global_original_argc (global()->original_argc)
#define global_original_argv (global()->original_argv)
#define global_other_owl_data (global()->other_owl_data)
#define global_outfilename (global()->outfilename)
#define global_output_flags (global()->output_flags)
#define global_owl_branch_depth (global()->owl_branch_depth)
#define global_owl_cache (global()->owl_cache)
#define global_owl_distrust_depth (global()->owl_distrust_depth)
#define global_owl_goal_worm (global()->owl_goal_worm)
#define global_owl_node_limit (global()->owl_node_limit)
#define global_owl_reading_depth (global()->owl_reading_depth)
#define global_owl_stack (global()->owl_stack)
#define global_owl_stack_pointer (global()->owl_stack_pointer)
#define global_owl_stack_size (global()->owl_stack_size)
#define global_owl_threats (global()->owl_threats)
#define global_pfd_a (global()->pfd_a)
#define global_pfd_b (global()->pfd_b)
#define global_play_mirror_go (global()->play_mirror_go)
#define global_play_out_aftermath (global()->play_out_aftermath)
#define global_position_number (global()->position_number)
#define global_posixly_correct (global()->posixly_correct)
#define global_potential_moves (global()->potential_moves)
#define global_prefer_ko (global()->prefer_ko)
#define global_printboard (global()->printboard)
#define global_printdragons (global()->printdragons)
#define global_printmoyo (global()->printmoyo)
#define global_printworms (global()->printworms)
#define global_profile_patterns (global()->profile_patterns)
#define global_quiet (global()->quiet)
#define global_rand_initialized (global()->rand_initialized)
#define global_reading_cache (global()->reading_cache)
#define global_replacement_map (global()->replacement_map)
#define global_resign_allowed (global()->resign_allowed)
#define global_result_certain (global()->result_certain)
#define global_rgoal (global()->rgoal)
#define global_routine_hash (global()->routine_hash)
#define global_s_worms (global()->s_worms)
#define global_semeai_branch_depth (global()->semeai_branch_depth)
#define global_semeai_branch_depth2 (global()->semeai_branch_depth2)
#define global_semeai_node_limit (global()->semeai_node_limit)
#define global_semeai_variations (global()->semeai_variations)
#define global_semeai_worms (global()->semeai_worms)
#define global_sgf_dump (global()->sgf_dump)
#define global_sgf_dumptree (global()->sgf_dumptree)
#define global_shadow (global()->shadow)
#define global_showscore (global()->showscore)
#define global_showstatistics (global()->showstatistics)
#define global_showtime (global()->showtime)
#define global_slowest_move (global()->slowest_move)
#define global_slowest_movenum (global()->slowest_movenum)
#define global_slowest_time (global()->slowest_time)
#define global_stack (global()->stack)
#define global_stackp (global()->stackp)
#define global_stats (global()->stats)
#define global_strategical_weight (global()->strategical_weight)
#define global_string (global()->string)
#define global_string_mark (global()->string_mark)
#define global_string_number (global()->string_number)
#define global_superstring_depth (global()->superstring_depth)
#define global_surround_pointer (global()->surround_pointer)
#define global_surroundings (global()->surroundings)
#define global_target1_hash (global()->target1_hash)
#define global_target2_hash (global()->target2_hash)
#define global_territorial_weight (global()->territorial_weight)
#define global_territory_cache_color (global()->territory_cache_color)
#define global_territory_cache_influence_id (global()->territory_cache_influence_id)
#define global_territory_cache_position_number (global()->territory_cache_position_number)
#define global_territory_correction (global()->territory_correction)
#define global_territory_determination_value (global()->territory_determination_value )
#define global_thrashing_dragon (global()->thrashing_dragon)
#define global_thrashing_stone (global()->thrashing_stone)
#define global_to_gnugo_stream (global()->to_gnugo_stream)
#define global_total_time (global()->total_time)
#define global_trymove_counter (global()->trymove_counter)
#define global_use_optimistic_territory (global()->use_optimistic_territory)
#define global_used_index (global()->used_index)
#define global_verbose (global()->verbose)
#define global_vertex_stack (global()->vertex_stack)
#define global_vertex_stack_pointer (global()->vertex_stack_pointer)
#define global_white_captured (global()->white_captured)
#define global_white_domain (global()->white_domain)
#define global_white_eye (global()->white_eye)
#define global_white_vital_points (global()->white_vital_points)
#define global_worm (global()->worm)
#define global_worms_examined (global()->worms_examined)
#define global_x (global()->x)
#define global_read_defend_db (global()->read_defend_db)
#define global_read_attack_db (global()->read_attack_db)
#define global_influencepat_db (global()->influencepat_db)
#define global_ttable (global()->ttable)
#define global_transformation (global()->transformation)
#define global_spiral (global()->spiral)
#define global_aftermath_sgftree (global()->aftermath_sgftree)
#define global_aftermath (global()->aftermath)
#define global_play_aftermath_current_board (global()->play_aftermath_current_board)
#define global_play_aftermath_current_color (global()->play_aftermath_current_color)
#define global_stones_on_board_stone_count_for_position (global()->stones_on_board_stone_count_for_position)
#define global_stones_on_board_white_stones (global()->stones_on_board_white_stones)
#define global_stones_on_board_black_stones (global()->stones_on_board_black_stones)
#define global_keyhash_init_is_initialized (global()->keyhash_init_is_initialized)
#define global_clk (global()->clk)
#define global_aa_status (global()->aa_status)
#define global_forbidden (global()->forbidden)
#define global_aa_values (global()->aa_values)
#define global_current_minsize (global()->current_minsize)
#define global_current_attacks (global()->current_attacks)
#define global_conditional_attack_point (global()->conditional_attack_point)
#define global_dragon_escape_mx (global()->dragon_escape_mx)
#define global_dragon_escape_mx_initialized (global()->dragon_escape_mx_initialized)
#define global_filllib_adjs (global()->filllib_adjs)
#define global_filllib_libs (global()->filllib_libs)
#define global_horizontally_symmetric (global()->horizontally_symmetric)
#define global_vertically_symmetric (global()->vertically_symmetric)
#define global_diagonally_symmetric (global()->diagonally_symmetric)
#define global_fuseki_moves (global()->fuseki_moves)
#define global_fuseki_value (global()->fuseki_value)
#define global_num_fuseki_moves (global()->num_fuseki_moves)
#define global_fuseki_total_value (global()->fuseki_total_value)
#define global_search_mask (global()->search_mask)
#define global_remaining_handicap_stones (global()->remaining_handicap_stones)
#define global_total_handicap_stones (global()->total_handicap_stones)
#define global_handicap_matches (global()->handicap_matches)
#define global_number_of_matches (global()->number_of_matches)
#define global_hash_is_initialized (global()->hash_is_initialized)
#define global_hash_white_hash (global()->hash_white_hash)
#define global_hash_black_hash (global()->hash_black_hash)
#define global_hash_ko_hash (global()->hash_ko_hash)
#define global_hashdata_to_string_buffer (global()->hashdata_to_string_buffer)
#define global_accumulate_influence_working (global()->accumulate_influence_working)
#define global_accumulate_influence_working_area_initialized (global()->accumulate_influence_working_area_initialized)
#define global_segment_region_marked (global()->segment_region_marked)
#define global_compute_escape_influence_cached_board (global()->compute_escape_influence_cached_board)
#define global_compute_escape_influence_escape_values (global()->compute_escape_influence_escape_values)
#define global_compute_escape_influence_active_caches (global()->compute_escape_influence_active_caches)
#define global_PROFILE_MATCHER_totals (global()->PROFILE_MATCHER_totals)
#define global_class_mask (global()->class_mask)
#define global_dfa_board_size (global()->dfa_board_size)
#define global_dfa_p (global()->dfa_p)
#define global_matchpat_num_stones (global()->matchpat_num_stones)
#define global_matchpat_pattern_stones (global()->matchpat_pattern_stones)
#define global_eyevalue_to_string_result (global()->eyevalue_to_string_result)
#define global_remembered_board_hashes (global()->remembered_board_hashes)
#define global_owl_defend_owl (global()->owl_defend_owl)
#define global_location_to_string_init (global()->location_to_string_init)
#define global_location_to_string_buf (global()->location_to_string_buf)
#define global_nodes_connect (global()->nodes_connect)
#define global_connection_shadow (global()->connection_shadow)
#define global_breakin_shadow (global()->breakin_shadow)
#define global_global_connection_node_counter (global()->global_connection_node_counter)
#define global_reading_node_counter (global()->reading_node_counter)
#define global_nodes_when_called (global()->nodes_when_called)
#define global_fast_defense_liberty_mark (global()->fast_defense_liberty_mark)
#define global_fast_defense_lm (global()->fast_defense_lm)
#define global_defend_lib_score (global()->defend_lib_score)
#define global_defend_not_adjacent_lib_score (global()->defend_not_adjacent_lib_score)
#define global_defend_capture_score (global()->defend_capture_score)
#define global_defend_atari_score (global()->defend_atari_score)
#define global_defend_save_score (global()->defend_save_score)
#define global_defend_open_score (global()->defend_open_score)
#define global_attack_own_lib_score (global()->attack_own_lib_score)
#define global_attack_string_lib_score (global()->attack_string_lib_score)
#define global_attack_capture_score (global()->attack_capture_score)
#define global_attack_save_score (global()->attack_save_score)
#define global_attack_open_score (global()->attack_open_score)
#define global_defend_not_edge_score (global()->defend_not_edge_score)
#define global_attack_not_edge_score (global()->attack_not_edge_score)
#define global_attack_ko_score (global()->attack_ko_score)
#define global_cannot_defend_penalty (global()->cannot_defend_penalty)
#define global_safe_atari_score (global()->safe_atari_score)
#define global_local_tree (global()->local_tree)
#define global_safe_move_initialized (global()->safe_move_initialized)
#define global_safe_move_cache (global()->safe_move_cache)
#define global_safe_move_cache_when (global()->safe_move_cache_when)
#define global_random_seed (global()->random_seed)
#define global_warned (global()->warned)
#define global_timers (global()->timers)
#define global_gg (global()->gg)
#define global_dragon_num (global()->dragon_num)
#define global_next_white (global()->next_white)
#define global_next_black (global()->next_black)
#define global_sgf_column (global()->sgf_column)
#define global_sgffile (global()->sgffile)
#define global_sgferr (global()->sgferr)
#define global_sgferrarg (global()->sgferrarg)
#define global_sgferrpos (global()->sgferrpos)
#define global_lookahead (global()->lookahead)
#define global_find_worm_threats_libs (global()->find_worm_threats_libs)
#define global_find_worm_attacks_and_defenses_libs (global()->find_worm_attacks_and_defenses_libs)
#define global_do_compute_effective_worm_sizes_worms (global()->do_compute_effective_worm_sizes_worms)
#define global_replacement_map (global()->replacement_map)
#define global_current_color (global()->current_color)
#define global_next_reason (global()->next_reason)
#define global_conn_worm1 (global()->conn_worm1)
#define global_conn_worm2 (global()->conn_worm2)
#define global_next_connection (global()->next_connection)
#define global_new_dragon_origins (global()->new_dragon_origins)


#ifdef __cplusplus
}
#endif

#endif
