#ifndef __BOARD_CONTROL_H__
#define __BOARD_CONTROL_H__

/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <coecntrl.h>
#include <coeinput.h>
class CgnugoAppView;
/*! 
  @class CBoardControl
  
  @discussion An instance of the Application View object for the gnugo 
  example application
  */
class CBoardControl : public CCoeControl
    {
public:

/*!
  @function NewL
   
  @discussion Create a CBoardControl object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CBoardControl
  */
    static CBoardControl* NewL(CgnugoAppView &aContainer, const TRect& aRect, class GoGame *game);

/*!
  @function NewLC
   
  @discussion Create a CBoardControl object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CBoardControl
  */
    static CBoardControl* NewLC(CgnugoAppView &aContainer, const TRect& aRect, class GoGame *game);


/*!
  @function ~CBoardControl
  
  @discussion Destroy the object and release all memory objects
  */
     ~CBoardControl();

	TPoint Vertex (TUint x, TUint y) const;

	class Player {
		public:
			enum Type {
				None,
				Black,
				White,
				Count
			};
	};

        void NewGame (int boardSize);

	void DrawCounter (TUint x, TUint y) const;
	void DrawDot (TUint x, TUint y) const;
	void DrawCursor (TUint x, TUint y, TBool aComputerCursor = EFalse) const;
	void SetCursor (TUint x, TUint y);
	void SetCounter (Player::Type player, TUint x, TUint y);

	virtual TKeyResponse OfferKeyEventL (const TKeyEvent &aKeyEvent, TEventCode aType);
	virtual TCoeInputCapabilities InputCapabilities() const;

	void SetPlayer (Player::Type player);
	Player::Type CyclePlayer (Player::Type player) const;
	Player::Type Counter (TUint x, TUint y) const;


public:  // from CCoeControl
/*!
  @function Draw
  
  @discussion Draw this CBoardControl to the screen
  @param aRect the rectangle of this view that needs updating
  */
    void Draw(const TRect& aRect) const;
  
protected: 
	virtual void SizeChanged();
	
    /**
     *  From CCoeControl
     *  Handles pointer events
     */	  
    void HandlePointerEventL(const TPointerEvent& aPointerEvent);
private:
/*!
  @function ConstructL
  
  @discussion  Perform the second phase construction of a CBoardControl object
  @param aRect the rectangle this view will be drawn to
	*/
    void ConstructL(CgnugoAppView &aContainer, const TRect& aRect, class GoGame *game);
	
    static void RedrawCallback (void *data);
	
	/*!
  @function CBoardControl
  
  @discussion Perform the first phase of two phase construction 
  */
    CBoardControl();

    TUint centeringX;
    TUint centeringY;

    TUint cellWidth;
    TUint cellHeight;

    TUint iBoardWidth;
    TUint iBoardHeight;

    TUint iCurrentX;
    TUint iCurrentY;

    Player::Type currentPlayer;

    class GoGame *iGame;

	CgnugoAppView* iMainWindow;
#ifdef UIQ3
	CFbsBitmap* iBoardBitmap;
	CFbsBitmap* iWhiteNodes[6];
	CFbsBitmap* iBlackNodes[6];
#endif

    };

#endif // __GNUGO_APPVIEW_H__
