#ifndef GoGame_h
#define GoGame_h

/*
GnuGoS60 - A Port of GNU Go to Series60/Symbian
Copyright (C) 2006, 2007  Ian Dunbar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
	
	  You should have received a copy of the GNU General Public License
	  along with this program; if not, write to the Free Software
	  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <e32base.h>
#include "gnugo.h"
#define HUMAN_HUMAN 0
#define HUMAN_CPU 1

class GoGameComputerWaiter;
class MGoGameObserver;

class GoGame:public CBase 
{
public:
	
	enum PlayerType {
			None,
			Black,
			White
		};

	enum TPlayState
	{
		ELoadedGameState,
		EPlayingState,
		EPassState,
		ECpuResignedState,
		EHumanResignedState
	};
	
	GoGame();
	~GoGame ();
	
	
	TUint BoardSize () const;
	void NewGame(TUint sz);
	void NewGame();
	void LoadGameL(const TDesC& aFileName, TBool aStartPlay);
	void SaveGameL(const TDesC& aFileName);
	void Resign();
	void Undo();
	bool IsOver () const;

	void EnterPlayL();
	void FirstMove();
	void LastMove();
	void NextMove();
	void PrevMove(); //?Undo

	TBool DoMove (TInt x, TInt y);
	bool IsHandicapSpot (TInt x, TInt y) const;
	PlayerType Vertex (TInt x, TInt y) const;
	void Pass ();
	bool IsGameOver () const;
	TBool ComputerThinking(){return iComputerThinking;};
	Gameinfo& GameInfo(){return iGameinfo;}
	
	void SetGoGameObserver (MGoGameObserver* aObserver);
	static TInt RunThreadL(TAny* aGoGame);
	void RunComputerThreadL();
	void ComputerMoveDone();
	TPlayState GameState();
	TDesC& StatusText();
	TPoint LastComputerMove();
	TInt iGameType;
	protected:
		void ComputerMove ();
		void SetHandicapSpots();
		void ReInit();
        void ReDraw();
        void init_sgf();
        TPlayState Endgame (TInt aReason);
		void UpdateStatusText(TInt aReason);
		void ParseNonMoveNode();
	private:			
		MGoGameObserver* iGoGameObserver;

        SGFTree iSgftree;
        Gameinfo iGameinfo;
        TUint iPasses;
        TPlayState iState;
        TInt iCurrent_score_estimate;
        TInt iResignation_allowed;
        char hspots[MAX_BOARD][MAX_BOARD];
        TInt iSgf_initialized;
        TInt iLast_move_i;      /* The position of the last move */
        TInt iLast_move_j;      /* -""-                          */
		TInt iLast_move_x;      /* The position of the last move */
        TInt iLast_move_y;      /* -""-                          */
		
		RThread iComputerThread;	
		TBool		iComputerThinking;
		TThreadId iMainThreadId;
		GoGameComputerWaiter* iComputerWaiter;
		TBuf<256> iStatusText;
		
		TInt human_color;

		/** Number of moves in the last loaded sgf */
		TInt iLoadedMoves;
};

class TGoGameResult
{
public:
	TReal iResult;
	float iUpper;
	float iLower;
	GoGame::PlayerType iWinner;
	GoGame::TPlayState iReason;
};

class MGoGameObserver
{
public:
	virtual void ReDrawL() = 0;
	virtual void BoardSizeChangedL() = 0;
	virtual void GameEndedL(TGoGameResult& aGameResult) = 0;
};


#endif
