#ifndef __GNUGO_APPVIEW_H__
#define __GNUGO_APPVIEW_H__

/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include <coecntrl.h>
#include <coeinput.h>
#include "GoGame.h"
/*! 
  @class CgnugoAppView
  
  @discussion An instance of the Application View object for the gnugo 
  example application
  */
#ifdef UIQ3
class CQikFullscreenFloatingMenu;
#include <qikviewbase.h>
class CgnugoAppView : public CQikViewBase, MGoGameObserver
#else
class CgnugoAppView : public CCoeControl, MGoGameObserver
#endif
    {
public:

/*!
  @function NewL
   
  @discussion Create a CgnugoAppView object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CgnugoAppView
  */
    static CgnugoAppView* NewL(const TRect& aRect, class GoGame *aGame);

/*!
  @function NewLC
   
  @discussion Create a CgnugoAppView object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CgnugoAppView
  */
    static CgnugoAppView* NewLC(const TRect& aRect, class GoGame *aGame);

	virtual TInt CountComponentControls() const;
	virtual CCoeControl *ComponentControl(TInt aIndex) const;

	virtual TKeyResponse OfferKeyEventL (const TKeyEvent &aKeyEvent, TEventCode aType);
	virtual TCoeInputCapabilities InputCapabilities() const;

/*!
  @function ~CgnugoAppView
  
  @discussion Destroy the object and release all memory objects
  */
     ~CgnugoAppView();

	void NewGame (TUint);
#ifdef UIQ3
	void ToggleFullScreen();
	void UpdatedCommandStateL();
#else
	virtual void HandleResourceChange(TInt aType);
#endif
protected: 
	virtual void SizeChanged();
	virtual void PositionChanged();
#ifndef UIQ3
/*!
  @function Draw
  
  @discussion Draw this CgnugoAppView to the screen
  @param aRect the rectangle of this view that needs updating
  */
    void Draw(const TRect& aRect) const;
#endif 
protected:
	virtual void ReDrawL();
	virtual void BoardSizeChangedL();
	virtual void GameEndedL(TGoGameResult& aGameResult);
private:
/*!
  @function ConstructL
  
  @discussion  Perform the second phase construction of a CgnugoAppView object
  @param aRect the rectangle this view will be drawn to
  */
    void ConstructL(const TRect& aRect, class GoGame *aGame);

/*!
  @function CgnugoAppView
  
  @discussion Perform the first phase of two phase construction 
  */
#ifdef UIQ3
    CgnugoAppView(CQikAppUi& aAppUi);
	TVwsViewId CgnugoAppView::ViewId() const;
	
	void ViewActivatedL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,const TDesC8& aCustomMessage);
	
	void ViewDeactivated();

	 // from CQikViewBase
	void ViewConstructL();
	void HandleCommandL(CQikCommand& aCommand);
public:
	void UpdateStatusTextL();
protected:
	CQikFullscreenFloatingMenu* iFloatingMenu;
#else
    CgnugoAppView();
#endif
    class CBoardControl *iBoardControl;

    class GoGame *iGame;

	TBool iFullScreen;
    };


#endif // __GNUGO_APPVIEW_H__
