#ifndef __GNUGO_DOCUMENT_H__
#define __GNUGO_DOCUMENT_H__

/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



// Forward references
class CgnugoAppUi;
class CEikApplication;


/*! 
  @class CgnugoDocument
  
  @discussion An instance of class CgnugoDocument is the Document part of the AVKON
  application framework for the gnugo example application
  */
#ifdef UIQ3
#include <qikdocument.h>
class CgnugoDocument : public CQikDocument
#else
#include <akndoc.h>
class CgnugoDocument : public CAknDocument
#endif
    {
public:

/*!
  @function NewL
  
  @discussion Construct a CgnugoDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp application creating this document
  @result a pointer to the created instance of CgnugoDocument
  */
    static CgnugoDocument* NewL(CEikApplication& aApp);

/*!
  @function NewLC
  
  @discussion Construct a CgnugoDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp application creating this document
  @result a pointer to the created instance of CgnugoDocument
  */
    static CgnugoDocument* NewLC(CEikApplication& aApp);

/*!
  @function ~CgnugoDocument
  
  @discussion Destroy the object and release all memory objects
  */
    ~CgnugoDocument();

public: // from CAknDocument
/*!
  @function CreateAppUiL 
  
  @discussion Create a CgnugoAppUi object and return a pointer to it
  @result a pointer to the created instance of the AppUi created
  */
    CEikAppUi* CreateAppUiL();

private:

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CgnugoDocument object
  */
    void ConstructL();

/*!
  @function CgnugoDocument
  
  @discussion Perform the first phase of two phase construction 
  @param aApp application creating this document
  */
    CgnugoDocument(CEikApplication& aApp);

    };


#endif // __GNUGO_DOCUMENT_H__
