/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <e32std.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <eikenv.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#define CPU_CAPTURED (iGameinfo.computer_player== WHITE ? black_captured : white_captured)
#define HUMAN_CAPTURED (human_color== WHITE ? black_captured : white_captured)

#include "liberty.h"

#include "gg-getopt.h"
#include "gg_utils.h"

#include "sgftree.h"
#include "random.h"

#include "Debug.h"
#include "GoGame.h"

#define NO_SCORE 4711

_LIT(szScore, "Score:");
_LIT(szCPUCaptured, "CPU ");
_LIT(szHumanCaptured, "You ");
_LIT(szWhiteCaptured, "White ");
_LIT(szBlackCaptured, "Black ");
_LIT(szPass, "pass");
_LIT(szUndo, "UNDO");
_LIT(szFormatA, "%S");
_LIT(szFormatB, "%d");

_LIT(szFormatPlayback, "%d/%d %S%d; %S%d");

_LIT(szFormatNorm, "(%c-%d) %S%d; %S%d");
_LIT(szFormatPass, "(%S) %S%d, %S%d");
_LIT(szFormatScore, "%S %S%.1f, %S%.1f (k=%.1f,H=%d)");
_LIT(szScoring, "Everything ok? If yes:Menu>Pass");
_LIT(szWelcome, "Welcome to Symbian GNU Go!");
//_LIT(szGnuGoThinks[4][40] = { TEXT("GNU Go thinks..."), TEXT("GNU Go contemplates your move..."), TEXT("GNU Go evaluates its position..."), TEXT("GNU Go is deep in thought...") };
_LIT(szCongrats, "Congrats!");
_LIT(szIWin, "I won!");
_LIT(szTie, "It's a tie!");
_LIT(szBlack, "B:");

class GoGameComputerWaiter:public CActive
{
public:
	GoGameComputerWaiter():CActive(0)
	{
		CActiveScheduler::Add(this);
		Start();
	}

	~GoGameComputerWaiter()
	{
		Cancel();
	}

	void Start()
	{
		iStatus = KRequestPending;
		SetActive();
	}

	void RunL()
	{
		iGame->ComputerMoveDone();
		Start();
	}

	void DoCancel()
	{
		TRequestStatus* status = &iStatus;
		User::RequestComplete(status, KErrCancel);
	}

	GoGame* iGame;
};

void GoGame::ComputerMoveDone()
{
	CEikonEnv::Static()->BusyMsgCancel();
	ReDraw();

	if (iPasses >= 2) { iState = Endgame (0); }						

}

bool GoGame::IsOver () const
{
    DEBUG_PRINTF (("GoGame::IsOver"));
    return ((iState != 0) || (iPasses >= 2));
}
void GoGame::SetGoGameObserver (MGoGameObserver* aObserver)
{
	iGoGameObserver = aObserver;
}

void GoGame::ReDraw ()
{
	if (iGoGameObserver) {
		iGoGameObserver->ReDrawL();
	}
}

TUint GoGame::BoardSize() const
{
    return board_size;
}
void GoGame::init_sgf()
{
    DEBUG_PRINTF (("GoGame::init_sgf"));
    if (iSgf_initialized)
        return;
    iSgf_initialized = 1;

    DEBUG_PRINTF (("GoGame::init_sgf 00"));
    sgf_write_header(iSgftree.root, 1, get_random_seed(), komi,
            level, chinese_rules);
    DEBUG_PRINTF (("GoGame::init_sgf 01"));
    sgfOverwritePropertyInt(iSgftree.root, "HA", iGameinfo.handicap);
    DEBUG_PRINTF (("GoGame::init_sgf 02"));
    if (iGameinfo.handicap > 0)
        gnugo_recordboard(iSgftree.root);
    DEBUG_PRINTF (("GoGame::init_sgf 03"));
}

void GoGame::Undo()
{
	TInt movenumber = gnugo_get_move_number();
	if ((movenumber > 0)&&(iState==EPlayingState)) {
		gnugo_undo_move(2);
		sgftreeBack(&iSgftree);
		iPasses = 0;
		iLast_move_x = iLast_move_y = iLast_move_i = iLast_move_j = -1;
		/*InvalidateRect(hWnd, NULL, TRUE); // refresh*/
		UpdateStatusText(0);
		ReDraw();
	}
}

TDesC& GoGame::StatusText()
{
	return iStatusText;
}

TPoint GoGame::LastComputerMove()
{
	return TPoint(iLast_move_x, iLast_move_y);
}

void GoGame::UpdateStatusText(TInt aReason)
{
	// update status text
	switch(aReason)
	{
	case 0: // undo
		if (iGameType == HUMAN_CPU) {
			iStatusText.Format(szFormatPass, &szUndo(),
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);
		} else {
			iStatusText.Format(szFormatPass, &szUndo(),
				&szBlackCaptured(), black_captured, &szWhiteCaptured(), white_captured);
		}
		break;
	case 1: // pass
		// generate status string for PASS
		if (iGameType == HUMAN_CPU) {
			iStatusText.Format( szFormatPass, &szPass(),
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);
		} else {
			iStatusText.Format( szFormatPass, &szPass(),
				&szBlackCaptured(), black_captured, &szWhiteCaptured(), white_captured);
		}
		
		break;
	case 2: // play		
		// generate status string for PLAY
		if (iGameType == HUMAN_CPU) {
			iStatusText.Format( szFormatNorm, (char)(iLast_move_i+'@'), (board_size-iLast_move_j)+1,
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);
		} else {
			iStatusText.Format( szFormatNorm,  (char)(iLast_move_i+'@'), (board_size-iLast_move_j)+1,
				&szBlackCaptured(), black_captured,& szWhiteCaptured(), white_captured);
		}
		break;
		
		
	}
}

void GoGame::Pass ()
{
    DEBUG_PRINTF (("GoGame::Pass"));
    iPasses++;
    init_sgf();
    gnugo_play_move(-1, -1, iGameinfo.to_move);
    sgffile_add_debuginfo(iSgftree.lastnode, 0);
    sgftreeAddPlay(&iSgftree, iGameinfo.to_move, -1, -1);
    sgffile_output(&iSgftree);

    iGameinfo.to_move = OTHER_COLOR(iGameinfo.to_move);

    if (0) {
        iGameinfo.computer_player = OTHER_COLOR(iGameinfo.computer_player);
        sgftreeAddComment(&iSgftree, "forced");
        return;
    }
	UpdateStatusText(1);

	if(iState == EPlayingState)
	{
		CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));
		iComputerThread.Resume();
	}
}

bool GoGame::IsGameOver () const
{
    DEBUG_PRINTF (("GoGame::IsGameOver"));
    return (iPasses >= 2);
}

void GoGame::LoadGameL(const TDesC& aFileName, TBool aStartPlay)
{
	Resign();
	ReInit();
	TBuf8<256> filename;
	filename.Copy(aFileName);
	filename.PtrZ();
	sgftree_readfile(&iSgftree, (const char*) filename.PtrZ());
	if(iSgftree.root != NULL)
	{
		iSgftree.lastnode = NULL;
		iGameinfo.game_record = iSgftree;
	
		iPasses = 0;
		iState = EPlayingState;
		iSgf_initialized = 1;
		/* to avoid changing handicap etc. */		
		gameinfo_load_sgfheader(&iGameinfo, iSgftree.root);
		SetHandicapSpots ();

		if(aStartPlay)
		{	
			iGameinfo.to_move = gameinfo_play_sgftree(&iGameinfo, &iSgftree, NULL);

			sgfOverwritePropertyInt(iSgftree.root, "HA", iGameinfo.handicap);
			
			/* Does the computer play first?  If so, make a move. */
			if (iGameinfo.computer_player == iGameinfo.to_move && iState == EPlayingState) 
			{
				
				iComputerThread.Resume();
				CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));	
			}
			iState = EPlayingState;
		}
		else
		{
			iState = ELoadedGameState;
			SGFNode* node = iSgftree.root;

			iLoadedMoves = 0;
			while(node != NULL)
			{
				if(is_move_node(node))
				{
					iLoadedMoves++;
				}

				node = node->child;
			}

			NextMove();
		}
		
		iGoGameObserver->BoardSizeChangedL();
		
		ReDraw();					
	}
	else
	{
		NewGame();
	}
}

void GoGame::ParseNonMoveNode()
{
	SGFProperty *sgf_prop;
	SGFNode* node = iSgftree.lastnode;
	/* If the node has no properties, there's nothing to do.
	This should have been checked by the caller, but it can't hurt. */
	if (!node->props)
		return;
	
	sgf_prop = node->props;
	char* prop_value;
	while (sgf_prop) {
		prop_value = NULL;
		switch (sgf_prop->name) {
		case SGFAN: /** Annotation*/
			{
			prop_value = sgf_prop->value;
			}
			break;
		case SGFC: // Comment
			{
				prop_value = sgf_prop->value;
			}
			break;
		case SGFCA: // Copyright
			{
				prop_value = sgf_prop->value;
			}
			break;
		case SGFCP: // Game 
			{ 
				prop_value = sgf_prop->value;
			}break;
		case SGFGC: // Game comment
			{
				prop_value = sgf_prop->value;
			}break;
		case SGFGN: // Game name
			{
				prop_value =  sgf_prop->value;
			}break;


		default:
			break;
		}
		sgf_prop = sgf_prop->next;

		// Add prop value to observer for information or other?
		if(prop_value != NULL)
		{
			int i = 0;
		}
	}
	
	
}

void GoGame::EnterPlayL()
{
	sgfOverwritePropertyInt(iSgftree.root, "HA", iGameinfo.handicap);
	iState = EPlayingState;

	/* Does the computer play first?  If so, make a move. */
	if (iGameinfo.computer_player == iGameinfo.to_move && iState == EPlayingState) 
	{
		
		iComputerThread.Resume();
		CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));	
	}
}
	
void GoGame::FirstMove()
{
	iStatusText.Format( szFormatPlayback, gnugo_get_move_number(), iLoadedMoves,
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);

	ReDraw();
}

void GoGame::LastMove()
{
	 while (sgftreeForward(&iSgftree))
           iGameinfo.to_move = gnugo_play_sgfnode(iSgftree.lastnode,
						  iGameinfo.to_move);

	 iStatusText.Format( szFormatPlayback, gnugo_get_move_number(), iLoadedMoves,
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);


	 ReDraw();
}

void GoGame::NextMove()
{
	// Get initial node in the sgf tree
	TInt forwardresult = 0;
	while((forwardresult = sgftreeForward(&iSgftree)) && !is_move_node(iSgftree.lastnode))
	{
		ParseNonMoveNode();
	}

	if (forwardresult && iSgftree.lastnode && is_move_node(iSgftree.lastnode)) {
		 iGameinfo.to_move = gnugo_play_sgfnode(iSgftree.lastnode,
						  iGameinfo.to_move);
	}

	iStatusText.Format( szFormatPlayback, gnugo_get_move_number(), iLoadedMoves,
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);

	ReDraw();
}

void GoGame::PrevMove()
{
	TInt movenumber = gnugo_get_move_number();
	if (movenumber > 0 && gnugo_undo_move(1)) {	
		sgftreeBack(&iSgftree);
		iGameinfo.to_move = OTHER_COLOR(iGameinfo.to_move);
	}

	iStatusText.Format( szFormatPlayback, gnugo_get_move_number(), iLoadedMoves,
				&szCPUCaptured(), CPU_CAPTURED, &szHumanCaptured(), HUMAN_CAPTURED);


	ReDraw();
}

void GoGame::SaveGameL(const TDesC& aFileName)
{
	TBuf8<256> filename;
	filename.Copy(aFileName);
	filename.PtrZ();
	/* make sure we are saving proper handicap */
	init_sgf();
	writesgf(iSgftree.root, (const char*) filename.PtrZ());
}


void GoGame::NewGame(TUint sz)
{
    DEBUG_PRINTF (("GoGame::NewGame"));
    Resign ();
    ReInit ();
    gnugo_clear_board(sz);
    iPasses = 0;
    iState = EPlayingState;

    /* Allow resignation at interface level (the engine may still be not
     * allowed to resign.
     */
    iResignation_allowed = 1;
    iCurrent_score_estimate = NO_SCORE;

	SetHandicapSpots ();
    sgfOverwritePropertyInt(iSgftree.root, "SZ", sz);
    sgfOverwritePropertyInt(iSgftree.root, "HA", iGameinfo.handicap);

	
    /* Set the game_record to be identical to the loaded one or the
     * newly created empty sgf tree.
     */
    iGameinfo.game_record = iSgftree;

	if (iGameinfo.handicap == 0) {

        iGameinfo.to_move = BLACK;

    } else {
		/* In case max handicap changes on smaller board. */
		iGameinfo.handicap = gnugo_placehand(iGameinfo.handicap);
        iGameinfo.to_move = WHITE;

    }

	iStatusText = szWelcome();

	ReDraw ();

    /* Does the computer play first?  If so, make a move. */
    if (iGameinfo.computer_player == iGameinfo.to_move && iState == EPlayingState) 
	{
		
		iComputerThread.Resume();
		CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));	
    }
}

void GoGame::NewGame()
{
    DEBUG_PRINTF (("GoGame::NewGame"));
    Resign ();
    ReInit ();
    gnugo_clear_board(board_size);

	iPasses = 0;
    iState = EPlayingState;

    /* Allow resignation at interface level (the engine may still be not
     * allowed to resign.
     */
    iResignation_allowed = 1;
    iCurrent_score_estimate = NO_SCORE;

	SetHandicapSpots ();
    /* In case max handicap changes on smaller board. */
    iGameinfo.handicap = gnugo_placehand(iGameinfo.handicap);
    sgfOverwritePropertyInt(iSgftree.root, "SZ", board_size);
    sgfOverwritePropertyInt(iSgftree.root, "HA", iGameinfo.handicap);

    /* Set the game_record to be identical to the loaded one or the
     * newly created empty sgf tree.
     */
    iGameinfo.game_record = iSgftree;

	iStatusText = szWelcome();

	if (iGameinfo.handicap == 0) {

        iGameinfo.to_move = BLACK;

    } else {

        iGameinfo.handicap = gnugo_placehand(iGameinfo.handicap);

        iGameinfo.to_move = WHITE;

    }

	ReDraw ();


    /* Does the computer play first?  If so, make a move. */
    if (iGameinfo.computer_player == iGameinfo.to_move && iState == EPlayingState) 
	{
		iComputerThread.Resume();
		CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));	
    }
}

void GoGame::Resign()
{
    DEBUG_PRINTF (("GoGame::Resign"));
	if(iSgftree.root != NULL)
	{
    int color = OTHER_COLOR(iGameinfo.to_move);

    sgftreeWriteResult(&iSgftree, color == WHITE ? 1000.0 : -1000.0, 1);

    iState = EHumanResignedState;
	}
}

void GoGame::ReInit()
{
    DEBUG_PRINTF (("GoGame::ReInit"));
    sgffile_output(&iSgftree);
    iPasses = 0;

    /* Play a different game next time. */
    update_random_seed();

    /* Free the sgf tree and prepare for a new game. */
    sgfFreeNode(iSgftree.root);
    sgftree_clear(&iSgftree);
    sgftreeCreateHeaderNode(&iSgftree, board_size, komi);
    iSgf_initialized = 0;
	Gameinfo oldInfo = iGameinfo;
    gameinfo_clear(&iGameinfo, board_size, komi);
	iGameinfo.computer_player = oldInfo.computer_player;
	iGameinfo.handicap = oldInfo.handicap;
	iLast_move_x = iLast_move_y = iLast_move_i = iLast_move_j = -1;
}

GoGame::GoGame ()
{
    DEBUG_PRINTF (("GoGame::GoGame"));

	iStatusText = szWelcome();
	iComputerWaiter = new (ELeave) GoGameComputerWaiter;
	iComputerWaiter->iGame = this;
    float memory = (float) DEFAULT_MEMORY; /* Megabytes used for hash table. */

	// Japanese rules default komi
    komi = 6.5;
	showscore = 1;
    level = DEFAULT_LEVEL;
	iGameType = HUMAN_CPU;
    min_level = 0;

    max_level = 10;

    mandated_depth               = -1;

    mandated_backfill_depth      = -1;

    mandated_backfill2_depth     = -1;

    mandated_break_chain_depth   = -1;

    mandated_superstring_depth   = -1;

    mandated_fourlib_depth       = -1;

    mandated_ko_depth            = -1;

    mandated_branch_depth        = -1;

    mandated_owl_distrust_depth  = -1;

    mandated_owl_branch_depth    = -1;

    mandated_owl_reading_depth   = -1;

    mandated_owl_node_limit      = -1;

    mandated_aa_depth            = -1;


    debug = 0;

    fusekidb = 1;

    disable_fuseki = 0;

    josekidb = 1;

	iGameinfo.computer_player = WHITE;

	iGameinfo.handicap = 0;

    chinese_rules = (CHINESE_RULES)? 1 : 0;
    owl_threats = (OWL_THREATS)? 1 : 0;


    experimental_connections = EXPERIMENTAL_CONNECTIONS;
    cosmic_gnugo = COSMIC_GNUGO;
    large_scale = LARGE_SCALE;
    resign_allowed = RESIGNATION_ALLOWED;


    allow_suicide = 0;
    capture_all_dead = 0;
    play_out_aftermath = 0;
    limit_search = 0;


    /* Default parameters for clock and auto level systems. */
    clock_init(3600, 0, 0);      /* One hour sudden death. */

   /* If seed is zero, GNU Go will play a different game each time. If
     * it is set using -r, GNU Go will play the same game each time.
     * (Change seed to get a different game).
     */
    int seed = 0;
    /* Start random number seed. */
    seed = time(0);


    /* Initialize the GNU Go engine. */
    init_gnugo(memory, seed);

	RThread thisThread;
	iMainThreadId = thisThread.Id();
	iComputerThread.Create(_L("GnuGoComputer"),
            RunThreadL,
            81920,
            NULL,
            this);
	
//	ReInit();
    //sgftree_clear(&iSgftree);

    //gameinfo_clear(&iGameinfo, board_size, komi);

//	gnugo_clear_board(board_size);
  //  SetHandicapSpots ();
	NewGame();

    /* Initialize and empty sgf tree if there was no infile. */
   // sgftreeCreateHeaderNode(&iSgftree, board_size, komi);


    /* Set the game_record to be identical to the loaded one or the
     * newly created empty sgf tree.
     */
  //  iGameinfo.game_record = iSgftree;


#if ORACLE
    if (metamachine) {

        summon_oracle();

        oracle_loadsgf(infilename, untilstring);

    }
#endif  
}

TInt GoGame::RunThreadL(TAny* aGoGame)
{
	static_cast<GoGame*>(aGoGame)->RunComputerThreadL();
	return KErrNone;
}

void GoGame::RunComputerThreadL()
{
	while(1)
	{
		if(iGameType == HUMAN_CPU)
		{
			iComputerThinking = ETrue;
			ComputerMove ();  
			iComputerThinking = EFalse;
		}

		RThread mainThread;		
		mainThread.Open(iMainThreadId);
		TRequestStatus* status = &iComputerWaiter->iStatus;
		mainThread.RequestComplete(status, KErrNone);
		mainThread.Close();
		
		
		RThread thisThread;
		thisThread.Suspend();
	}
}

TBool GoGame::DoMove (TInt x, TInt y)
{
    DEBUG_PRINTF (("GoGame::DoMove"));
	if (!gnugo_is_legal(x-1, y-1, iGameinfo.to_move)) {
		return EFalse;
	}

	if(iState == EPlayingState)
	{
		
		iPasses = 0;
		iLast_move_i = x;
		iLast_move_j = y;
		DEBUG_PRINTF (("GoGame::DoMove 00"));
		gnugo_play_move(x-1, y-1, iGameinfo.to_move);
		sgffile_add_debuginfo(iSgftree.lastnode, 0);
		sgftreeAddPlay(&iSgftree, iGameinfo.to_move, iLast_move_i-1, iLast_move_j-1);
		sgffile_output(&iSgftree);
		
		DEBUG_PRINTF (("GoGame::DoMove 01"));
		iGameinfo.to_move = OTHER_COLOR(iGameinfo.to_move);
		UpdateStatusText(2);
		ReDraw ();
		
		DEBUG_PRINTF (("GoGame::DoMove 02"));
		if(iState == EPlayingState)
		{
			iComputerThread.Resume();
			CEikonEnv::Static()->BusyMsgL(_L("Thinking..."));
		}
		return ETrue;
	}

	return EFalse;
}

GoGame::TPlayState GoGame::GameState()
{
	return iState;
}

GoGame::TPlayState GoGame::Endgame (TInt aReason)
{
    DEBUG_PRINTF (("GoGame::EndGame"));
	TPlayState newState = EPlayingState;
    if (aReason == 0) {
        /* Two iPasses, game is over. */
        //gnugo_who_wins(iGameinfo.computer_player, stdout);
		// printf("\nIf you disagree, we may count the game together.\n");
		TGoGameResult goResult;
		goResult.iReason = EPassState;
		goResult.iResult = gnugo_estimate_score(&goResult.iUpper, &goResult.iLower);
		if (goResult.iResult == 0.0)
			goResult.iWinner = GoGame::None;
		else 
			goResult.iWinner = (goResult.iResult > 0.0)?GoGame::White : GoGame::Black;
		
		goResult.iResult = abs(goResult.iResult);
        sgftreeWriteResult(&iSgftree, estimate_score(NULL, NULL), 1);
		newState = EPassState;

		if(iGoGameObserver != NULL)
		{
			iGoGameObserver->GameEndedL(goResult);
		}
    } else {
        int color = OTHER_COLOR(iGameinfo.to_move);
		TGoGameResult goResult;
		goResult.iResult = 1000;
        if (aReason == 1)		// Our opponent has resigned. 
		{
			goResult.iReason = ECpuResignedState;
		}
        else			// We have resigned. 
		{
			goResult.iReason = EHumanResignedState;        
		}

		goResult.iWinner = color == WHITE ? GoGame::White: GoGame::Black;

        sgftreeWriteResult(&iSgftree, color == WHITE ? 1000.0 : -1000.0, 1);
		newState = 	goResult.iReason;
		
		if(iGoGameObserver != NULL)
		{
			iGoGameObserver->GameEndedL(goResult);
		}
    }

    return newState;
}

void GoGame::ComputerMove ()
{
    DEBUG_PRINTF (("GoGame::ComputerMove"));
    int i, j;
    int move_val;
    int resignation_declined = 0;
    float upper_bound, lower_bound;

    DEBUG_PRINTF (("GoGame::ComputerMove"));
    init_sgf();

    /* Generate computer move. */
    DEBUG_PRINTF (("GoGame::ComputerMove 00"));
    move_val = gnugo_genmove(&i, &j, iGameinfo.to_move);


    /* computer has resigned */
    if (iResignation_allowed && move_val < 0.0 && ON_BOARD2(i, j)) {
    DEBUG_PRINTF (("GoGame::ComputerMove 01"));
        int iState = Endgame(2);
        if (iState != -1)
            return;//  iState;
    DEBUG_PRINTF (("GoGame::ComputerMove 02"));

        /* The opponent declined resignation. Remember not to resign again. */
        iResignation_allowed = 0;
        resignation_declined = 1;
        move_val = -move_val;

        return;
    }


    DEBUG_PRINTF (("GoGame::ComputerMove 03"));
    if (showscore) {

    DEBUG_PRINTF (("GoGame::ComputerMove 04"));
        gnugo_estimate_score(&upper_bound, &lower_bound);

    DEBUG_PRINTF (("GoGame::ComputerMove 05"));
        iCurrent_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
    DEBUG_PRINTF (("GoGame::ComputerMove 06"));

    }
    iLast_move_x = i;
    iLast_move_y = j;

    if (is_pass(POS(i, j))) {
        iPasses++;
		iLast_move_x = -1;
		iLast_move_y = -1;
		UpdateStatusText(1);
    } else {
        iPasses = 0;
    }
    DEBUG_PRINTF (("GoGame::ComputerMove 07"));

    gnugo_play_move(i, j, iGameinfo.to_move);

    DEBUG_PRINTF (("GoGame::ComputerMove 08"));
    sgffile_add_debuginfo(iSgftree.lastnode, move_val);
    DEBUG_PRINTF (("GoGame::ComputerMove 09"));
    sgftreeAddPlay(&iSgftree, iGameinfo.to_move, i, j);
    DEBUG_PRINTF (("GoGame::ComputerMove 10"));
    if (resignation_declined)
        sgftreeAddComment(&iSgftree, "GNU Go resignation was declined");
    DEBUG_PRINTF (("GoGame::ComputerMove 11"));
	 sgffile_output(&iSgftree);

	if(iPasses==0)
	{
		UpdateStatusText(2);
	}
    iGameinfo.to_move = OTHER_COLOR(iGameinfo.to_move);
    DEBUG_PRINTF (("GoGame::ComputerMove 12"));

	
    iState = EPlayingState;
}

bool GoGame::IsHandicapSpot (int x, int y) const
{
    x--;
    y--;
    if (x < 0) return false;
    if (y < 0) return false;
    if (x >= board_size) return false;
    if (y >= board_size) return false;
    return ('+' == hspots[x][y]);
}

GoGame::PlayerType GoGame::Vertex (TInt x, TInt y) const
{

    switch (BOARD(x-1,y-1))
    {
        case BLACK:
            return GoGame::Black;
        case WHITE:
            return GoGame::White;
        default:
            return GoGame::None;
    }
}

GoGame::~GoGame ()
{
#if ORACLE

    dismiss_oracle();

#endif

    clock_report_autolevel(NULL, iGameinfo.computer_player);
	destroy_gnugo();
	iComputerThread.Terminate(KErrNone);
	delete iComputerWaiter;
}

void GoGame::SetHandicapSpots()
{
    DEBUG_PRINTF (("GoGame::SetHandicapSpots"));

    memset(hspots, '.', sizeof(hspots));


    if (board_size == 5) {

        /* place the outer 4 */
        hspots[1][1] = '+';
        hspots[board_size-2][1] = '+';
        hspots[1][board_size-2] = '+';
        hspots[board_size-2][board_size-2] = '+';
        /* and the middle one */
        hspots[board_size/2][board_size/2] = '+';
        return;
    }


    if (!(board_size%2)) {

        /* If the board size is even, no center handicap spots. */
        if (board_size > 2 && board_size < 12) {

            /* Place the outer 4 only. */
            hspots[2][2] = '+';
            hspots[board_size-3][2] = '+';
            hspots[2][board_size-3] = '+';
            hspots[board_size-3][board_size-3] = '+';
        }
        else {

            /* Place the outer 4 only. */
            hspots[3][3] = '+';
            hspots[board_size-4][3] = '+';
            hspots[3][board_size-4] = '+';
            hspots[board_size-4][board_size-4] = '+';
        }
    }
    else {

        /* Uneven board size */
        if (board_size > 2 && board_size < 12) {

            /* Place the outer 4... */
            hspots[2][2] = '+';
            hspots[board_size-3][2] = '+';
            hspots[2][board_size-3] = '+';
            hspots[board_size-3][board_size-3] = '+';

            /* ...and the middle one. */
            hspots[board_size/2][board_size/2] = '+';
        }
        else {

            /* Place the outer 4... */
            hspots[3][3] = '+';
            hspots[board_size-4][3] = '+';
            hspots[3][board_size-4] = '+';
            hspots[board_size-4][board_size-4] = '+';

            /* ...and the inner 4... */
            hspots[3][board_size/2] = '+';
            hspots[board_size/2][3] = '+';
            hspots[board_size/2][board_size-4] = '+';
            hspots[board_size-4][board_size/2] = '+';

            /* ...and the middle one. */
            hspots[board_size/2][board_size/2] = '+';
        }
    }

}

