/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef UIQ3
#include <qikappui.h>
#include <eikdialg.h>
#include <gnugo.rsg>
#include <QikSelectFileDlg.h>
#include <qikselectfolderdlg.h>
#include <QikSaveFileDlg.h>
#else
#include <avkon.hrh>
#include <aknnotewrappers.h> 
#include <eikbtgpc.h>
#include <avkon.rsg>
#endif

#include "gnugo.pan"
#include "gnugoAppUi.h"
#include "gnugoAppView.h"
#include "gnugo.hrh"
#include "Debug.h"
#include "GoGame.h"
#include <eikdialg.h>
_LIT(KSGFExt,".sgf");
class CGnuGoNewGameDialog:public CEikDialog
{
public:
	CGnuGoNewGameDialog(GoGame& aGoGame);
	~CGnuGoNewGameDialog();
protected:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aCommandId);
private:
	GoGame& iGoGame;
};

#include <eikchlst.h>
#include <qikfloatingpointeditor.h>
#include <eikchkbx.h>
CGnuGoNewGameDialog::CGnuGoNewGameDialog(GoGame& aGoGame):iGoGame(aGoGame)
{
}

CGnuGoNewGameDialog::~CGnuGoNewGameDialog()
{
}

void CGnuGoNewGameDialog::PreLayoutDynInitL()
{
	CEikChoiceList* playerColor =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGamePlayerColor));
	
	switch(iGoGame.GameInfo().computer_player)
	{
	default:
	case WHITE:
		playerColor->SetCurrentItem(0);
		break;
	case BLACK:
		playerColor->SetCurrentItem(1);
		break;		
	}

	CEikLabeledCheckBox * againstPdaBox = static_cast<CEikLabeledCheckBox *>(Control(EGnuGoNewGameAgainstPDA));
	againstPdaBox->SetState(iGoGame.iGameType==HUMAN_CPU? CEikButtonBase::ESet:CEikButtonBase::EClear);

	CEikChoiceList* boardSize =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameBoardSize));
	switch(iGoGame.BoardSize())
	{
	case 19:
		boardSize->SetCurrentItem(0);
		break;
	case 13:
		boardSize->SetCurrentItem(1);
		break;
	case 9:
		boardSize->SetCurrentItem(2);
		break;
	}

	CEikChoiceList* levelCtrl =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameLevel));
	levelCtrl->SetCurrentItem(level);


	CQikFloatingPointEditor* komiCtrl = static_cast<CQikFloatingPointEditor*>(Control(EGnuGoNewGameKomi));
	komiCtrl->SetValueL(komi);
	
	CEikChoiceList* handicap =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameBlackHandicap));
	handicap->SetCurrentItem(iGoGame.GameInfo().handicap);
}

TBool CGnuGoNewGameDialog::OkToExitL(TInt /*aCommandId*/)
{
	CEikChoiceList* playerColor =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGamePlayerColor));
	iGoGame.GameInfo().computer_player = playerColor->CurrentItem()==0?WHITE:BLACK;

	CEikLabeledCheckBox* againstPdaBox = static_cast<CEikLabeledCheckBox*>(Control(EGnuGoNewGameAgainstPDA));
	iGoGame.iGameType = againstPdaBox->State()==CEikButtonBase::ESet?HUMAN_CPU:HUMAN_HUMAN;

	CEikChoiceList* boardSize =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameBoardSize));
	switch(boardSize->CurrentItem())
	{
	case 0:
		board_size = 19;
		break;
	case 1:
		board_size = 13;
		break;
	case 2:
		board_size = 9;
		break;
	}

	CEikChoiceList* levelCtrl =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameLevel));
	level = levelCtrl->CurrentItem();

	CQikFloatingPointEditor* komiCtrl = static_cast<CQikFloatingPointEditor*>(Control(EGnuGoNewGameKomi));
	komi = komiCtrl->Value();
	
	CEikChoiceList* handicap =  static_cast<CEikChoiceList*>(Control(EGnuGoNewGameBlackHandicap));
	iGoGame.GameInfo().handicap =handicap->CurrentItem();
	return ETrue;
}



class CGnuGoOptionsDialog:public CEikDialog
{
public:
	CGnuGoOptionsDialog(GoGame& aGoGame);
	~CGnuGoOptionsDialog();
protected:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aCommandId);
private:
	GoGame& iGoGame;
};
#include <eiklabel.h>
#include <QikNumberEditor.h>

CGnuGoOptionsDialog::CGnuGoOptionsDialog(GoGame& aGoGame):iGoGame(aGoGame)
{
}

CGnuGoOptionsDialog::~CGnuGoOptionsDialog()
{
}

void CGnuGoOptionsDialog::PreLayoutDynInitL()
{
	TCoeFont font(12, TCoeFont::EPlain);
	static_cast<CEikLabel*>(Control(EGnuGoOptExlain))->SetFont(font);


	CQikNumberEditor* depthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptDepth));
	depthCtrl->SetValueL(mandated_depth);

	CQikNumberEditor* fourLibDepthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptFourLibDepth));
	fourLibDepthCtrl->SetValueL(mandated_fourlib_depth);

	CQikNumberEditor* backFillCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptBackFill));
	backFillCtrl->SetValueL(mandated_backfill_depth);
	
	CQikNumberEditor* koDepthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptKoDepth));
	koDepthCtrl->SetValueL(mandated_ko_depth);
}

TBool CGnuGoOptionsDialog::OkToExitL(TInt /*aCommandId*/)
{

	CQikNumberEditor* depthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptDepth));
	mandated_depth = depthCtrl->Value();
	
	CQikNumberEditor* fourLibDepthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptFourLibDepth));
	mandated_fourlib_depth = depthCtrl->Value();
	
	CQikNumberEditor* backFillCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptBackFill));
	mandated_backfill_depth = backFillCtrl->Value();

	CQikNumberEditor* koDepthCtrl = static_cast<CQikNumberEditor*>(Control(EGnuGoOptKoDepth));
	mandated_ko_depth = koDepthCtrl->Value();

	return ETrue;
}

class  CGnuGoSaveNameDlg:public CEikDialog
{
public:
	CGnuGoSaveNameDlg(TFileName& aFileName);
	~CGnuGoSaveNameDlg();
protected:
	void PreLayoutDynInitL();
	TBool OkToExitL(TInt aCommandId);
private:
	TFileName& iFileName;
};

#include <eikedwin.h>
CGnuGoSaveNameDlg::CGnuGoSaveNameDlg(TFileName& aFileName):iFileName(aFileName)
{
}

CGnuGoSaveNameDlg::~CGnuGoSaveNameDlg()
{
}

void CGnuGoSaveNameDlg::PreLayoutDynInitL()
{
}

TBool CGnuGoSaveNameDlg::OkToExitL(TInt /*aCommandId*/)
{
	TParsePtr parser(iFileName);
	TBuf<64> filename;
	iFileName = parser.DriveAndPath();
	static_cast<CEikEdwin*>(ControlOrNull(0x1000))->GetText(filename);
	iFileName.Append(filename);	

	if(parser.Ext().Length() == 0)
	{
		iFileName.Append(KSGFExt());
	}

	return ETrue;
}


// ConstructL is called by the application framework
void CgnugoAppUi::ConstructL()
{
        DEBUG_PRINTF (("CgnugoAppUi::ConstructL"));
	BaseConstructL();

	//CEikStatusPane *sp = StatusPane();
	//sp->SwitchLayoutL (R_AVKON_STATUS_PANE_LAYOUT_USUAL_FLAT);
	iGame = new (ELeave) GoGame ();
	DEBUG_PRINTF (("after gogame!"));

	iAppView = CgnugoAppView::NewL(ClientRect(), iGame);    
#ifdef UIQ3
	AddViewL(*iAppView);
#else
	AddToStackL(iAppView);
#endif
}

CgnugoAppUi::CgnugoAppUi()                              
{
        DEBUG_PRINTF (("CgnugoAppUi::CgnugoAppUi"));

	// no implementation required
}

CgnugoAppUi::~CgnugoAppUi()
{
	DEBUG_PRINTF (("~CgnugoAppUi::~CgnugoAppUi"));
	if (iGame) { delete iGame; iGame = 0; }
	
#ifndef UIQ3
	if (iAppView)
	{
		RemoveFromStack(iAppView);
		delete iAppView;
		iAppView = NULL;
	}
#endif

}

// handle any menu commands
void CgnugoAppUi::HandleCommandL(TInt aCommand)
{
    DEBUG_PRINTF (("CgnugoAppUi::HandleCommandL"));
    switch(aCommand)
    {
        case EEikCmdExit:
#ifndef UIQ3
        case EAknSoftkeyExit:
#endif
            Exit();
            break;
		case EgnugoCommandNewGame:
			{
				CGnuGoNewGameDialog* dlg = new (ELeave) CGnuGoNewGameDialog(*iGame);
				if(dlg->ExecuteLD(R_GNUGO_NEWGAME_DLG))
				{
					iAppView->NewGame(iGame->BoardSize());
				}

			}break;
		case EGnugoCommandLoadAndPlay:
		case EgnugoCommandLoad:
			{
#ifdef UIQ3
				//TFileName fileName;
				CDesCArrayFlat* mimeArray = new (ELeave) CDesCArrayFlat(5);
				CleanupStack::PushL(mimeArray);
				CDesCArrayFlat* fileArray = new (ELeave) CDesCArrayFlat(5);
				CleanupStack::PushL(fileArray);
				if(CQikSelectFileDlg::RunDlgLD(*mimeArray, *fileArray))
				{
					if(fileArray->MdcaCount()>0)
					{
					iGame->LoadGameL(fileArray->MdcaPoint(0), aCommand == EGnugoCommandLoadAndPlay);				
					}
				}
				CleanupStack::PopAndDestroy(2, mimeArray);
#else
#endif
			}break;
		case EgnugoCommandSave:
			{
#ifdef UIQ3
			TFileName fileName;
			CQikSelectFolderDlg::RunDlgLD(fileName, NULL);
			CEikDialog* saveNameDlg = new (ELeave) CGnuGoSaveNameDlg(fileName);
			if(saveNameDlg->ExecuteLD(R_GNUGO_SAVENAME_DLG))
			{
				iGame->SaveGameL(fileName);
			}
#else
#endif
			}break;
		case EgnugoCommandEnterGame:
			iGame->EnterPlayL();
			break;
		case EgnugoCommandFirstMove:
			iGame->FirstMove();
			break;
		case EgnugoCommandNextMove:
			iGame->NextMove();
			break;
		case EgnugoCommandPrevMove:
			iGame->PrevMove();
			break;
		case EgnugoCommandLastMove:
			iGame->LastMove();
			break;
        case EgnugoCommandNewGame19x19:
            {
                iAppView->NewGame (19);
            }
            break;
        case EgnugoCommandNewGame13x13:
            {
                iAppView->NewGame (13);
            }
            break;
        case EgnugoCommandNewGame9x9:
            {
                iAppView->NewGame (9);
            }
            break;
		case EgnugoCommandUndo:
			{
				iGame->Undo();							
			}break;
        case EgnugoCommandPass:
            {
                iGame->Pass ();
            }
            break;
	
		case EgnugoCommandOptions:
			{				
				CGnuGoOptionsDialog* dlg = new (ELeave) CGnuGoOptionsDialog(*iGame);
				dlg->ExecuteLD(R_GNUGO_OPTIONS_DLG);
			}break;

		case EgnugoCommandHowTo:
			{				
				CEikDialog* dlg = new (ELeave) CEikDialog;
				dlg->ExecuteLD(R_GNUGO_HELP_DLG);
			}break;

		case EgnugoCommandAbout:
			{
				CEikDialog* dlg = new (ELeave) CEikDialog;
				dlg->ExecuteLD(R_GNUGO_ABOUT_DLG);
			/*	dlg = new (ELeave) CEikDialog;
				dlg->ExecuteLD(R_GNUGO_GAMEOVER_DLG);*/									
			}break;

        case EgnugoCommandFullscreen:
            {
#ifdef UIQ3
				iAppView->ToggleFullScreen();
#else
                StatusPane()->MakeVisible(EFalse);
#endif
            }
            break;

        default:
            Panic(EgnugoBasicUi);
            break;
    }
}

// -----------------------------------------------------------------------------
// CFilelistAppUi::HandleResourceChangeL( TInt aType )
// Called by framework when layout is changed.
// -----------------------------------------------------------------------------
//
void CgnugoAppUi::HandleResourceChangeL( TInt aType )
{
        DEBUG_PRINTF (("CgnugoAppUi::HandleResourceChangeL"));
#ifdef UIQ3
	CQikAppUi::HandleResourceChangeL( aType );
#else
	CAknAppUi::HandleResourceChangeL( aType );

	if ( aType==KEikDynamicLayoutVariantSwitch )
	{
		// Change the status pane to use the small status pane
		//CEikStatusPane *sp = StatusPane();
		//sp->SwitchLayoutL (R_AVKON_STATUS_PANE_LAYOUT_USUAL_FLAT);

		// Update the child controls for the new layout
		iAppView->SetRect( ClientRect() );
	}
#endif
	// allow the child controls to handle the change
	iAppView->HandleResourceChange( aType );
}

void CgnugoAppUi::HandleStatusPaneSizeChange ()
{
        DEBUG_PRINTF (("CgnugoAppUi::HandleStatusPaneSizeChange"));
	// Need to manually notify the child controls that their size has changed when the status pane changes
	if (iAppView) iAppView->SetRect( ClientRect() );
}
