/*
    GnuGoS60 - A Port of GNU Go to Series60/Symbian
    Copyright (C) 2006, 2007  Ian Dunbar

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <gnugo.rsg>
#include "gnugo.hrh"
#include "GoGame.h"
#ifdef UIQ3
#include <gnugo.mbg>
#include <qikappui.h>
#include <qikviewbase.h>
#include <qikcommand.h>
#include <QikFullscreenFloatingMenu.h>
#else
#include <aknutils.h> 
#include <aknviewappui.h> 
#include <aknappui.h> 
#include <aknnotewrappers.h> 
#endif
#include "BoardControl.h"
#include "gnugoAppView.h"

// Standard construction sequence
CgnugoAppView* CgnugoAppView::NewL(const TRect& aRect, class GoGame *aGame)
{
	CgnugoAppView* self = CgnugoAppView::NewLC(aRect, aGame);
	CleanupStack::Pop(self);
	return self;
}

CgnugoAppView* CgnugoAppView::NewLC(const TRect& aRect, class GoGame *aGame)
{
#ifdef UIQ3
	CgnugoAppView* self = new (ELeave) CgnugoAppView(static_cast<CQikAppUi&>(*CEikonEnv::Static()->EikAppUi()));
#else
	CgnugoAppView* self = new (ELeave) CgnugoAppView;
#endif
	CleanupStack::PushL(self);
	self->ConstructL(aRect, aGame);
	return self;
}

#ifdef UIQ3
CgnugoAppView::CgnugoAppView(CQikAppUi& aAppUi):CQikViewBase(aAppUi, KNullViewId),
#else
CgnugoAppView::CgnugoAppView():
#endif
iBoardControl (0)
{
	// no implementation required
}

CgnugoAppView::~CgnugoAppView()
{
	if (iBoardControl)
	{
		delete iBoardControl;
		iBoardControl = 0;
	}
#ifdef UIQ3
	 if(iFloatingMenu)
        {
        // The custom command operator must be removed from the command manager. 
        CQikCommandManager::Static(*iCoeEnv).RemoveCustomOperator(*this,*iFloatingMenu);

        iFloatingMenu = NULL; // No need to delete the member, it is in the CCoeControlArray.
        }
#endif
}

void CgnugoAppView::NewGame(TUint boardSize)
{
    iBoardControl->NewGame (boardSize);
}
void CgnugoAppView::ConstructL(const TRect& aRectArea, class GoGame *aGame)
{
	TRect rect = aRectArea;
#ifdef UIQ3
	BaseConstructL();
	if(rect.Size().iHeight<rect.Size().iWidth)
	{
	//	rect.SetHeight(rect.Size().iWidth);
	}
#endif
    iGame = aGame;
    iGame->SetGoGameObserver (this);

	// Create a window for this application view
	CreateWindowL();
	
	// Set the windows size
	SetRect(rect);

	// Create the board control
	TRect boardRect;
	if (rect.Width () > rect.Height ()) {
		boardRect.SetWidth (rect.Height ());
		boardRect.SetHeight (rect.Height ());
	} else {
		boardRect.SetWidth (rect.Width ());
		boardRect.SetHeight (rect.Width ());
	}
	iBoardControl = CBoardControl::NewL (*this, boardRect, iGame);

	// Activate the window, which makes it ready to be drawn
	ActivateL();
}

#ifndef UIQ3
// Draw this application's view to the screen
void CgnugoAppView::Draw(const TRect& /*aRect*/) const
{
	// Get the standard graphics context 
	CWindowGc& gc = SystemGc();

	// Just to make sure don't draw outside of the control
	gc.SetClippingRect (Rect ());

	// Set the origin to the top left of the control
	gc.SetOrigin (Rect().iTl);

	// Clear the view area
	gc.Clear ();
}

// ---------------------------------------------------------
// CgnugoAppView::HandleResourceChange()
// Called by framework when layout is changed.
// ---------------------------------------------------------
//
void CgnugoAppView::HandleResourceChange( TInt aType )
{
	CCoeControl::HandleResourceChange( aType );
	if ( aType==KEikDynamicLayoutVariantSwitch )
	{
		TRect rect = Rect ();

		if (rect.Width () > rect.Height ()) {
			rect.SetWidth (rect.Height ());
		} else {
			rect.SetHeight (rect.Width ());
		}
		if (iBoardControl) iBoardControl->SetRect (rect);
	}
	if (iBoardControl) iBoardControl->HandleResourceChange (aType);
}
#endif

void CgnugoAppView::PositionChanged( )
{
#ifdef UIQ3
	CQikViewBase::PositionChanged ();
#else
	CCoeControl::PositionChanged ();
#endif

	if (iBoardControl)
	{
		TPoint position = TPoint(0,0);
		position.iX+=((Size().iWidth/2)-(iBoardControl->Size().iWidth/2));
		position.iY+=((Size().iHeight/2)-(iBoardControl->Size().iHeight/2));
		
		iBoardControl->SetPosition (position);
	}
}

void CgnugoAppView::SizeChanged( )
{
#ifdef UIQ3
	CQikViewBase::SizeChanged ();
#else
	CCoeControl::SizeChanged ();
#endif
	TSize size = Size ();

	if (size.iWidth > size.iHeight) {
		size.iWidth = size.iHeight;
	} else {
		size.iHeight = size.iWidth;
	}
	if (iBoardControl) iBoardControl->SetSize (size);
	SetPosition(Position());
}

TInt CgnugoAppView::CountComponentControls() const
{
	return 1;
}

CCoeControl *CgnugoAppView::ComponentControl(TInt /*aIndex*/) const
{
	return iBoardControl;
}

TCoeInputCapabilities CgnugoAppView::InputCapabilities() const
{
	TCoeInputCapabilities capabilities (TCoeInputCapabilities::ENavigation);

	return capabilities;
}

TKeyResponse CgnugoAppView::OfferKeyEventL (const TKeyEvent &aKeyEvent, TEventCode aType)
{
	TKeyResponse response = EKeyWasNotConsumed;

	if (iBoardControl) response = iBoardControl->OfferKeyEventL (aKeyEvent, aType);

	return response;
}
#ifdef UIQ3
TVwsViewId CgnugoAppView::ViewId() const
{
	TVwsViewId view(TUid::Uid(0xA0001C1D),TUid::Uid(1));
	return view;
}

void CgnugoAppView::ViewActivatedL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,const TDesC8& aCustomMessage)
{
	MakeVisible(ETrue);
	CQikViewBase::ViewActivatedL(aPrevViewId,aCustomMessageId,aCustomMessage);
	if(aPrevViewId.iAppUid.iUid != 0 && aPrevViewId.iAppUid != TUid::Uid(0xA0001C1D))
	{
		SetParentView(aPrevViewId);
	}
}

void CgnugoAppView::ViewDeactivated()
{
	MakeVisible(EFalse);
	CQikViewBase::ViewDeactivated();
}

// from CQikViewBase
void CgnugoAppView::ViewConstructL() {
	ViewConstructFromResourceL(R_GNUGO_UI_CONFIGURATIONS);

	iFloatingMenu = CQikFullscreenFloatingMenu::NewL(*this);

    // Append the control to the CCoeCOntrolArray
    AddControlLC(iFloatingMenu);        // iFloatingMenu is pushed on the CleanupStack

    // The custom command operator must be added to the command manager.    
    CQikCommandManager::Static(*iCoeEnv).AddCustomOperatorL(*this,*iFloatingMenu);

    CleanupStack::Pop(iFloatingMenu);
	TCoeFont smallFont(12,TCoeFont::EBold);
	ViewContext()->AddTextL(0x3000, _L("Welcome to Symbian GNU Go!"),EHLeftVCenter, smallFont);

	UpdatedCommandStateL();
}

void CgnugoAppView::UpdateStatusTextL()
{
	ViewContext()->ChangeTextL(0x3000, iGame->StatusText());
}

void CgnugoAppView::UpdatedCommandStateL()
{
	CQikCommandManager& comMan = CQikCommandManager::Static();
	switch(iGame->GameState())
	{
	case GoGame::ELoadedGameState:
		{
			comMan.SetInvisible(*this, EgnugoCommandNextMove, EFalse);
			comMan.SetInvisible(*this, EgnugoCommandPrevMove, EFalse);
			comMan.SetInvisible(*this, EgnugoCommandFirstMove, EFalse);
			comMan.SetInvisible(*this, EgnugoCommandLastMove, EFalse);
			comMan.SetInvisible(*this, EgnugoCommandEnterGame, EFalse);
			
			comMan.SetInvisible(*this, EgnugoCommandUndo, ETrue);
			comMan.SetInvisible(*this, EgnugoCommandPass, ETrue);
		}
		break;
	default:
		{
			comMan.SetInvisible(*this, EgnugoCommandNextMove, ETrue);
			comMan.SetInvisible(*this, EgnugoCommandPrevMove, ETrue);
			comMan.SetInvisible(*this, EgnugoCommandFirstMove, ETrue);
			comMan.SetInvisible(*this, EgnugoCommandLastMove, ETrue);
			comMan.SetInvisible(*this, EgnugoCommandEnterGame, ETrue);
			
			comMan.SetInvisible(*this, EgnugoCommandUndo, EFalse);
			comMan.SetInvisible(*this, EgnugoCommandPass, EFalse);
		}
		break;
	}
}

void CgnugoAppView::HandleCommandL(CQikCommand& aCommand)
{
	if(aCommand.Id()>=EgnugoCommandNewGame && aCommand.Id()<EgnugoCommandLast)
	{
		iEikonEnv->EikAppUi()->HandleCommandL(aCommand.Id());
		UpdatedCommandStateL();
	}
	else
	{
		CQikViewBase::HandleCommandL(aCommand);
	}
}

void CgnugoAppView::ReDrawL()
{
	iBoardControl->DrawNow();
	UpdateStatusTextL();
}

void CgnugoAppView::BoardSizeChangedL()
{
	SetSize(Size());
}

void CgnugoAppView::GameEndedL(TGoGameResult& aGameResult)
{
	switch(aGameResult.iReason)
	{
	default:
	case GoGame::EPassState:
		{
			if(aGameResult.iResult == 0.0f)
			{
				iEikonEnv->InfoWinL(_L("Game result"),_L("The game is jigo!"));
			}
			else
			{
				TBuf<256> gameResult;
				switch(aGameResult.iWinner)
				{
				case GoGame::Black:
					gameResult.Format(_L("Black wins with the result +%.1f"),aGameResult.iResult);
					break;
				case GoGame::White:
					gameResult.Format(_L("White wins with the result +%.1f"),aGameResult.iResult);
					break;
				}
				iEikonEnv->InfoWinL(_L("Game result"), gameResult);
			}
		}
		break;
	case GoGame::EHumanResignedState:
		iEikonEnv->InfoWinL(_L("Game ended"),_L("GNU Go wins by resignation."));
		break;
	case GoGame::ECpuResignedState:
		iEikonEnv->InfoWinL(_L("Game ended"),_L("You win by resignation."));
		break;
	}
}

void CgnugoAppView::ToggleFullScreen()
{
	iFullScreen = !iFullScreen;
	 TQikViewMode viewmode;
	if(iFullScreen)
	{
	viewmode.SetFullscreen();
	}
	else
	{
	viewmode.SetNormal();
	}

	SetViewModeL(viewmode);
	DrawNow();
}


#endif

